#ifndef _SOCKETPP_H
#	define _SOCKETPP_H

#include <kernel/OS.h>

#include <net/netdb.h>

enum tcp_status_t {
	CLOSED=0, OPEN
};

#define TCP_INIT		'TCPi'
#define TCP_CONNECTED	'TCPc'
#define TCP_RECVDATA	'TCPr'
#define TCP_SENDDATA	'TCPs'
#define TCP_NOCONN		'TCPx'
#define TCP_ERR			'TCPe'

#define TCP_SEND_BRK	'TCPb'
#define TCP_SEND_AYT	'TCPa'
#define TCP_SEND_OOB	'TCPo'

#define DATA_ARG		"data"
#define HANDLER_ARG		"handler"
#define HOST_ARG		"host"
#define PORT_ARG		"port"

/*
 * If something replies with B_ERROR, check this arg to find the problem.
 */
#define ERROR_ARG		"error"

/*
 * Various messages to go along with ERROR_ARG
 */
#define NO_LOOPER_MSG	0
#define NO_HOST_MSG		1
#define NO_PORT_MSG		2
#define NO_SOCKET		3
#define NO_CONNECT		4
#define NO_HOST			5
#define SYS_ERR_BASE	128
#define SYS_ERR_MSG		(0+SYS_ERR_BASE)
#define NO_THREAD_MSG	(1+SYS_ERR_BASE)
#define NO_MEM_MSG		(2+SYS_ERR_BASE)

class Socket;
class TCPSocket;
class TCPLooper;

class Socket {
protected:
	int fd;
	char hostname[64];
	struct hostent *host_info;
	struct servent *service_info;
	struct sockaddr_in host_socket;
	tcp_status_t status;

public:
	Socket();
	~Socket();
	
	tcp_status_t get_status();
	char *get_hostname();
	struct hostent *get_hostent();
	
	virtual int init(const char *dest_host, const char *dest_port) = 0;
	virtual int32 read(char *buffer, int32 buf_len) = 0;
	virtual int32 write(char *buffer, int32 buf_len) = 0;
	virtual int32 write(char c) = 0;
};

class TCPSocket: public Socket {
public:
	TCPSocket();

	virtual int init(const char *dest_host, const char *dest_port);
	virtual int32 read(char *buffer, int32 buf_len);
	virtual int32 write(char *buffer, int32 buf_len);
	virtual int32 write(char c);

	virtual bool block_read(char *buffer, int32 buf_len);
	virtual bool block_write(char *buffer, int32 buf_len);
};

class TCPLooper: public BLooper {
private:
	thread_id socket_read;
	BHandler *msg_handler;
	TCPSocket *tcp_socket;

	static long read_thread_entry(void *arg);
	long read_thread();

public:
	TCPLooper();
	~TCPLooper();

	virtual void MessageReceived(BMessage *msg);

	virtual void InitConnection();
	virtual void SendLocal(void *data, int32 nbytes);
	virtual void SendRemote(void *data, int32 nbytes);
};

#endif /* _SOCKETPP_H */