#include <sys/types.h>
#include <ctype.h>
#include <string.h>
#include "amanda.h"
#include "arglist.h"

/* =================================================================== */
/* argument globbing */

/*
** Returns the number of arguments in a string passed in.  Doesn't do anywhere
** near as much globbing as system() would, but system fork()s and wait()s
** itself, and invokes /bin/sh
*/
int
EXECnumargs(buf)
    char *buf;
{
    char quot=0;
    int pos, tot;

    tot=0;
    pos=0;
    while (buf[pos]) {
        if ((buf[pos] == '"')||(buf[pos] == '\'')) {
            quot=buf[pos++];
            while (buf[pos]&&(buf[pos] != quot)) {
		if (buf[pos] == '\\')
		    pos++;
		if (buf[pos])
                    pos++;
	    }
            tot++;
        } else if (isspace(buf[pos])) {
            while (buf[pos]&&isspace(buf[pos]))
                pos++;
        } else {
            while (buf[pos]&&(!isspace(buf[pos]))) {
		if (buf[pos] == '\\')
                    pos++;
		if (buf[pos])
		    pos++;
	    }
            tot++;
        }
    }
    return (tot);
}

/*
** Fills in the logical arguments in cmdline into the double array called
** args
*/

void
EXECfillargs(args, cmdline)
    char **args;
    char *cmdline;
{
    char *pos;
    char quot;
    char *store;
    int done=0;

    store=(char*)malloc(strlen(cmdline)+1);
    strcpy(store, cmdline);
    cmdline=store;

    pos=cmdline;
    while (*pos) {
	while (*cmdline&&isspace(*cmdline))
	    cmdline++;
	pos=cmdline;
	if ((*pos=='"')||(*pos=='\'')) {
	    cmdline++;
	    quot=*pos++;
	    while (*pos&&(*pos!=quot)) {
		/* backslash escaping */
		if ((*pos == '\\') && (quot != '\''))
		    pos++;
		if (*pos)
		    pos++;
	    }
	} else
	    while (*pos&&(!isspace(*pos))) {
		if (*pos == '\\')
		    pos++;
		if (*pos)
		    pos++;
	    }
	if (!*pos)
	    done=1;
	*pos=0;
	*args=(char*)malloc(strlen(cmdline)+1);
	strcpy(*args, cmdline);
	if (!done)
	    cmdline=++pos;
	args++;
    }
    *args=0;
    free((void*)store);
}

/*
** create a two-dimensional array from va_list passed in
*/
#ifdef STDC_HEADERS
char **EXECargify(char *arg1, ...)
#else
char **EXECargify(arg1, va_alist)
char *arg1;
va_dcl
#endif
{
    char **argv = 0, **hold;
    int numargs = 0;
    va_list ap;

    if (!arg1) {
	argv = (char**)malloc(sizeof(char*));
	argv[0] = 0;
	return(argv);
    }
    numargs = 1;
    argv = (char**)malloc((numargs+1)*sizeof(char*));
    argv[0] = stralloc(arg1);

    arglist_start(ap, arg1);            /* setup argv */
    while (arg1) {
	arg1 = arglist_val(ap, char *);
	if (arg1) {
	    numargs++;
	    hold = (char**)realloc(argv, (numargs+1)*sizeof(char*));
	    if (hold) {
		argv = hold;
		argv[numargs-1] = stralloc(arg1);
	    } else {
		argv[numargs] = 0;
		return(argv);
	    }
	}
    }
    arglist_end(ap);
    argv[numargs] = 0;
    return(argv);
}

/*
** free a two-dimensional argument list
*/
void EXECargfree(char **list)
{
    int i;

    for (i = 0; list[i]; i++)
	free((void*)list[i]);
    free((void*)list);
}

/* =================================================================== */
/* text substitution */

typedef struct _SUBSTkeylist {
    char *name;
    char *value;
    struct _SUBSTkeylist *next;
} SUBSTkeylist;

static SUBSTkeylist *globkeylist = 0;

/* find the string associated with an identifier */
SUBSTkeylist *
SUBSTfindvalue(name)
    char *name;
{
    SUBSTkeylist *iter = globkeylist;
    for (iter = globkeylist; iter; iter = iter->next) {
	if (!strcasecmp(iter->name, name))
	    return(iter);
    }
    return(0);
}

/* add an identifier to the list */
void
SUBSTaddkey(name, value)
    char *name, *value;
{
    SUBSTkeylist *new;

    new = SUBSTfindvalue(name);
    if (new && new->value)
	free((void*)new->value);
    else if (!new) {
	new = (SUBSTkeylist*)malloc(sizeof(SUBSTkeylist));
	new->next = globkeylist;
	globkeylist = new;
	new->name = (char*)malloc(strlen(name)+1);
	strcpy(new->name, name);
    }
    new->value = (char*)malloc(strlen(value)+1);
    strcpy(new->value, value);
}

/*
** changes an identifier to a string
*/
void
SUBSTgetvalue(arg, ret, len)
    char *arg, *ret;
    size_t len;
{
    SUBSTkeylist *val;

    val = SUBSTfindvalue(arg);
    if (!val) {
	*ret = 0;
	return;
    }
    strncpy(ret, val->value, len);
}

/*
** gets the length of the string an identifier would yield
*/
size_t
SUBSTgetlength(arg)
    char *arg;
{
    SUBSTkeylist *val;

    val = SUBSTfindvalue(arg);
    if (!val || !val->value)
	return(0);
    return(strlen(val->value));
}

/*
** Takes a bunch of text and performs neat substitution on it.
*/
void
SUBSTchangetext(rtext, ret, len)
    char *rtext, *ret;
    size_t len;
{
    char *close;
    char *text;

    text=(char*)malloc(sizeof(char)*(strlen(rtext)+1));
    if (!text) {
	*ret = 0;
	return;
    }
    strcpy(text, rtext);
    rtext = text;
    while (*text&&(len>0)) {
	if ((*text=='<')&&(*(text-1)!='\\')) {
	    if ((*++text)=='%') {
		close=strchr(text, '>');
		text++;
		*close=0;
		SUBSTgetvalue(text, ret, len);
		len-= strlen(ret);
		ret+= strlen(ret);
		text=close+1;
	    }
	}
	if (*text) {
	    len--;
	    *ret++=*text++;
	}
    }
    if (!len)
	ret--;
    *ret=0;
    free(rtext);
}

/*
** Takes a bunch of text and figures out how long changetext would be
*/
size_t
SUBSTlengthtext(rtext)
    char *rtext;
{
    char *close;
    char *text;
    size_t len;

    text=(char*)malloc(sizeof(char)*(strlen(rtext)+1));
    if (!text)
        return(0);
    strcpy(text, rtext);
    rtext = text;
    len = 0;
    while (*text) {
        if ((*text=='<')&&(*(text-1)!='\\')) {
            if ((*++text)=='%') {
                close=strchr(text, '>');
                text++;
                *close=0;
                len += SUBSTgetlength(text);
                text=close+1;
            }
        }
        len++;
        text++;
    }
    free(rtext);
    len++;
    return(len);
}
