#include <string.h>
#include "configdata.h"
#include "misc.h"
#include "macros.h"

/* for the flock */
#if	defined(LOCK_NONE)
#include <sys/stat.h>
#endif /* defined(LOCK_NONE) */
#if	defined(LOCK_FLOCK)
#include <sys/file.h>
#endif	/* defined(LOCK_FLOCK) */
#if	defined(LOCK_LOCKF)
#if	defined(DO_HAVE_UNISTD)
#include <unistd.h>
#endif	/* defined(DO_HAVE_UNISTD) */
#include <fcntl.h>
#endif  /* defined(LOCK_LOCKF) */
#if	defined(LOCK_FCNTL)
#include <fcntl.h>
#if	defined(SEEK_SET)
#define SEEK_SET        0
#endif	/* !defined(SEEK_SET) */
#endif	/* defined(LOCK_FCNTL) */

/*
** This is a utility function for getting to the meat of a line of text.
** It returns a pointer to the first non-blank character in the line,
** or null if it is a comment or empty line.
*/
char *
MISCparseline(buf)
    char *buf;
{
    char *hash;

    /* Skip past whitespace and empty lines */
    while (buf[0]&&((buf[0]==' ')||(buf[0]=='\t')))
        buf++;
    hash = strchr(buf, '#');
    if (hash)
        *hash = 0;
    if (!buf[0]||(buf[0]=='\n'))
        return(0);
    return(buf);
}

/* STRgetparam */
/* Search a string of the form F1(args),F2(args) for item F1, return the
 * appropriate arguments.
 * Returns 1 if it found the proper item, 0 if not */
int
STRgetparam(prefix, str, ret, len)
    char *prefix, *str, *ret;
    size_t len;
{
    char *iter, *lparen, *rparen;
    int found, nest;

    iter = str;
    found = 0;
    while (!found && *iter) {
        if (*iter == ',') {
            iter++;
            continue;
        }
        if (!(lparen = strchr(iter, '('))) {
            /* check for parameterless field */
            if (rparen = strchr(iter, ','))
                *rparen = 0;
        } else {
            if (!(rparen = STRfindrparen(lparen+1)))
                return 0;
            *lparen = 0;
        }
        if (!strcmp(prefix, iter)) {
            /* got a match! store the value */
            *rparen = 0;
            strncpy(ret, lparen+1, len);
            ret[len] = 0;
            found = 1;
        } else
            /* skip past stuff beloning to this field */
            iter = rparen+1;
        if (lparen) {
            *lparen = '(';
            *rparen = ')';
        } else if (rparen)
            *rparen = ',';
    }
    if (!found)
        *ret = 0;
    return (found);
}

char *
STRfindrparen(str)
    char *str;
{
    int in = 1;
    char *iter;

    iter = str;
    while (*iter && in) {
        if (*iter == '(')
            in++;
        if (*iter == ')')
            in--;
        iter++;
    }
    if (in)
        return 0;
    return (iter-1);
}

/*
** Returns the number of arguments in a string passed in.  Doesn't do anywhere
** near as much globbing as system() would, but system fork()s and wait()s
** itself
*/
int
EXECnumargs(buf)
    char *buf;
{
    char quot=0;
    int pos, tot;

    tot=0;
    pos=0;
    while (buf[pos]) {
        if ((buf[pos] == '"')||(buf[pos] == '\'')) {
            quot=buf[pos++];
            while (buf[pos]&&(buf[pos] != quot))
                pos++;
            tot++;
        } else if (isspace(buf[pos])) {
            while (buf[pos]&&isspace(buf[pos]))
                pos++;
        } else {
            while (buf[pos]&&(!isspace(buf[pos])))
                pos++;
            tot++;
        }
    }
    return (tot);
}

/*
** Fills in the logical arguments in cmdline into the double array called
** args
*/

void
EXECfillargs(args, cmdline)
    char **args;
    char *cmdline;
{
    char *pos;
    char quot;
    char *store;
    int done=0;

    store=(char*)malloc(strlen(cmdline)+1);
    strcpy(store, cmdline);
    cmdline=store;

    pos=cmdline;
    while (*pos) {
        while (*cmdline&&isspace(*cmdline))
            cmdline++;
        pos=cmdline;
        if ((*pos=='"')||(*pos=='\'')) {
            cmdline++;
            quot=*pos++;
            while (*pos&&(*pos!=quot))
                pos++;
        } else
            while (*pos&&(!isspace(*pos)))
                pos++;
        if (!*pos)
            done=1;
        *pos=0;
        *args=(char*)malloc(strlen(cmdline)+1);
        strcpy(*args, cmdline);
        if (!done)
            cmdline=++pos;
        args++;
    }
    *args=0;
    free((void*)store);
}

static char delim[] = "{}!(),=:; \t\r\n\"";
static char blank[] = " \t\r\n";

int
PARSEindelim(ch, list)
    char ch;
    char *list;
{
    char *test = list;

    while (*test) {
        if (ch == *test)
            return(1);
        test++;
    }
    return(0);
}

char *
PARSEgettoken(str, ret)
    char *str;
    char *ret;
{
    char *ind;

    while (PARSEindelim(*str, blank))
        str++;
    if (!*str)
        return(0);
    if (PARSEindelim(*str, delim)) {
        ret[1] = 0;
        ret[0] = *str;
        return(str+1);
    }
    ind = str;
    while (*ind && !PARSEindelim(*ind, delim))
        ind++;
    ret[ind-str] = 0;
    strncpy(ret, str, ind-str);
    while (*ind && PARSEindelim(*ind, blank))
        ind++;
    return(ind);
}

/* functions that used to be in nnrpd, but had to move here so that I could
 * use them with rnews */

/*
**  Parse a string into a NULL-terminated array of words; return number
**  of words.  If argvp isn't NULL, it and what it points to will be
**  DISPOSE'd.
*/
int
LIMargify(line, argvp)
    char                *line;
    char                ***argvp;
{
    register char       **argv;
    register char       *p;
    register int        i;

    if (*argvp != NULL) {
        DISPOSE(*argvp[0]);
        DISPOSE(*argvp);
    }

    /*  Copy the line, which we will split up. */
    while (ISWHITE(*line))
        line++;
    i = strlen(line);
    p = NEW(char, i + 1);
    (void)strcpy(p, line);

    /* Allocate worst-case amount of space. */
    for (*argvp = argv = NEW(char*, i + 2); *p; ) {
        /* Mark start of this word, find its end. */
        for (*argv++ = p; *p && !ISWHITE(*p); )
            p++;
        if (*p == '\0')
            break;

        /* Nip off word, skip whitespace. */
        for (*p++ = '\0'; ISWHITE(*p); )
            p++;
    }
    *argv = NULL;
    return argv - *argvp;
}

/*
**  Match a list of newsgroup specifiers against a list of newsgroups.
**  func is called to see if there is a match.
*/
int
LIMpermmatch(Pats, list)
    char                **Pats;
    char                **list;
{
    register int        i;
    register char       *p;
    register int	match;

    if (Pats[0] == NULL)
        return 1;

    match = FALSE;
    for ( ; *list; list++)
        for (i = 0; (p = Pats[i]) != NULL; i++) {
            if (p[0] == '!') {
                if (wildmat(*list, ++p))
                    match = 0;
            }
            else if (wildmat(*list, p))
                match = 1;
        }

    return match;
}

/*
**  Parse a newsgroups line, return TRUE if there were any.
*/
BOOL
LIMnggetlist(argvp, list)
    char                ***argvp;
    char                *list;
{
    register char       *p;

    for (p = list; *p; p++)
        if (*p == ',')
            *p = ' ';

    return LIMargify(list, argvp) != 0;
}

/* Routine to lock the database. If you're not locking on an nfs-mounted
 * volume, or if your nfs locking works (it doesn't), the flock() call
 * provided is sufficient. Otherwise you'll need some sort of nfs-safe
 * (link-based) lock routine.
 */
int
lockfile(f)
  int f;
{
#if     defined(LOCK_FLOCK)
  return flock(f, LOCK_EX);
#endif  /* defined(LOCK_FLOCK) */

#if     defined(LOCK_LOCKF)
  return lockf(f, F_LOCK, 0L);
#endif  /* defined(LOCK_LOCKF) */

#if     defined(LOCK_FCNTL)
  struct flock        fl;

  fl.l_type = F_WRLCK;
  fl.l_whence = SEEK_SET;
  fl.l_start = 0;
  fl.l_len = 0;
  return fcntl(f, F_SETLKW, &fl);
#endif  /* defined(LOCK_FCNTL) */
}

/* Unlock the database. See comments above on lockfile(). */
void
unlockfile(f)
  int f;
{
#if     defined(LOCK_FLOCK)
  flock(f, LOCK_UN);
#endif  /* defined(LOCK_FLOCK) */

#if     defined(LOCK_LOCKF)
  lockf(f, F_ULOCK, 0L);
#endif  /* defined(LOCK_LOCKF) */

#if     defined(LOCK_FCNTL)
  struct flock        fl;

  fl.l_type = F_UNLCK;
  fl.l_whence = SEEK_SET;
  fl.l_start = 0;
  fl.l_len = 0;
  fcntl(f, F_SETLK, &fl);
#endif  /* defined(LOCK_FCNTL) */
}
