#include <sys/types.h>
#include <sys/select.h>
#include <unistd.h>
#include <stdio.h>
#include "filter.h"
#include "misc.h"

typedef struct _FILTlist {
    FILT *data;
    time_t last;
    struct _FILTlist *next;
} FILTlist;

typedef int (*CMDfunc)(char*, char*, void*, int*);

typedef struct _CMD {
    char *cmdname;
    CMDfunc cmd;
} CMD;

FILTlist *connlist = 0;

typedef void (*CMDpromptfunc)(void*);

int doarticle(char*, char*, void*, int*);
int doquit(char*, char*, void*, int*);
int doline(char*, CMD*, void*, int*);
int getuser(char*, char*, void*, int*);
int getremote(char*, char*, void*, int*);
void promptuser(void*);
void inloop(CMD*, CMDpromptfunc, char*, void*);

int
main(argc, argv)
    int argc;
    char *argv[];
{
    char cmd[256];
    CMD post[] =
     { {"POST", doarticle},
       {"QUIT", doquit},
       {0, 0} };

    inloop(post, 0, "02 Ready.\n", 0);
    return(0);
}

int
getuser(name, args, filt, done)
    char *name, *args;
    void *filt;
    int *done;
{
    FILTlist *iter;
    char user[256], *tail;

    tail = PARSEgettoken(args, user);
    if (!tail || (*tail))
        return(-1);
    if (!connlist) {
        connlist = (FILTlist*)malloc(sizeof(FILTlist));
        connlist->next = 0;
        iter = connlist;
    } else {
        for (iter = connlist; iter; iter = iter->next)
            if (!strcmp(user, FILTgetuser(iter->data))) {
                *((FILT**)filt) = iter->data;
                *done = 1;
                return(0);
            }
        iter = (FILTlist*)malloc(sizeof(FILTlist));
        iter->next = connlist;
        connlist = iter;
    }
    iter->data = FILTconnect(0);
    FILTsetuser(iter->data, user);
    iter->last = time(0);
    *((FILT**)filt) = iter->data;

    *done = 1;
    return(0);
}
/*
void
rem_getheader(filt, name)
    FILT *filt;
    char *name;
{
    char buf[256];
    sprintf(buf, "0023 %s", name);
    CMD send[] =
     { {"SEND", getarticle},
       {"QUIT", doquit},
       {0, 0} };
}
*/
int
doarticle(name, args, ignored, done)
    char *name, *args;
    void *ignored;
    int *done;
{
    FILT *filt;
    CMD user[] =
     { {"USER", getuser},
       {"QUIT", doquit},
       {0, 0} };

    inloop(user, promptuser, 0, (void*)&filt);
    FILTclearheaders(filt);
    return(0);
}

void
promptuser(ignored)
    void *ignored;
{
    printf("001 Send user\n");
}

int
doquit(name, args, ignored, done)
    char *name, *args;
    void *ignored;
    int *done;
{
    *done = 1;
    return(0);
}

/* loops over input until we get a signal indicating done, or EOF. */
void
inloop(cmds, func, prompt, data)
    CMD *cmds;
    CMDpromptfunc func;
    char *prompt;
    void *data;
{
    char cmd[256];
    int done;
    int res;

    if (func) 
        func(data);
    else
        puts(prompt);
    done = 0;
    while (!done && fgets(cmd, sizeof(cmd), stdin)) {
        if (cmd[strlen(cmd)-1] != '\n') {
            printf("9 Line too long.\n");
            while (!strchr(cmd, '\n') && fgets(cmd, sizeof(cmd), stdin))
                ;
            continue;
        }
        res = doline(cmd, cmds, data, &done); 
        if (res < 0)
            printf("9 Command not understood.\n");
        else
            printf("01 ack.\n");
        if (!done) {
            if (func)
                func(data);
            else
                printf(prompt);
        }
    }
}

/* takes a command line and calls a function associated with the first
 * command on the line */
int
doline(line, cmds, data, done)
    char *line;
    CMD *cmds;
    void *data;
    int *done;
{
    char buf[256], *iter;
    int index;

    iter = PARSEgettoken(line, buf);
    if (!iter)
        return;
    for (index = 0; cmds[index].cmdname; index++) {
        if (!strcasecmp(cmds[index].cmdname, buf))
            break;
    }
    if (cmds[index].cmdname) {
        return(cmds[index].cmd(cmds[index].cmdname, iter, data, done));
    }
    return(-1);
}
