#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include "clibrary.h"
#include "configdata.h"
#include "paths.h"
#include "track.h"
#include "limitfile.h"
#include "error.h"
#include "misc.h"
#include "filter.h"
#include "catchup.h"
#include "proto.h"
#include "subst.h"

#if	defined(DO_FILT_POST)
static char *LIMpostarr[] = {"P", "Post", 0};
#endif
#if	defined(DO_FILT_CROSSPOST)
static char *LIMcrosspostarr[] = {"C", "Crosspost", 0};
#endif
#if	defined(DO_FILT_FOLLOWUP)
static char *LIMfollowuparr[] = {"F", "Follow-up", 0};
#endif
#if	defined(DO_FILT_NOTIFY)
static char *LIMnotifyarr[] = {"N", "Notify", 0};
#endif
#if	defined(DO_FILT_GROUP)
static char *LIMgrouparr[] = {"G", "Groups", 0};
#endif
#if	defined(DO_FILT_EMILY)
static char *LIMemilyarr[] = {"E", "Emily", 0};
#endif

LIMent TRACKparams[] =
 {
#if	defined(DO_FILT_POST)
   { LIMpostarr,	LIMpost,	LIMbreidbart},
#endif
#if	defined(DO_FILT_CROSSPOST)
   { LIMcrosspostarr,	LIMcrosspost,	LIMcrossblock},
#endif
#if	defined(DO_FILT_FOLLOWUP)
   { LIMfollowuparr,	LIMcrosspost,	LIMcrossblock},
#endif
#if	defined(DO_FILT_NOTIFY)
   { LIMnotifyarr,	LIMnotify,	0},
#endif
#if	defined(DO_FILT_GROUP)
   { LIMgrouparr,	LIMcrossgroup,	LIMgroupblock},
#endif
#if	defined(DO_FILT_EMILY)
   { LIMemilyarr,	LIMemily,	LIMpostnews},
#endif
   { 0, 0, 0 }
 };

void
PROTOdelcharpp(type, data, len)
    int type;
    void *data;
    size_t len;
{
    char **real = (char**)data;
    int index;

    for (index = 0; real[index]; index++)
        free((void*)real[index]);
}

void
TRACKinit()
{
    FILTadddel(kFILTcharpptype, PROTOdelcharpp);
    LIMsetparams(TRACKparams, sizeof(TRACKparams)/sizeof(LIMent)-1);
}

#if	defined(DO_FILT_POST)
int
LIMpost(filt, name, param)
    FILT *filt;
    char *name, *param;
{
    char *comma;
    int slow, stop;
    char buf[10];

    if (!param || !(*param))
        return(0);
    if (!(comma = strchr(param, ',')))
        return(0);
    *comma = 0;
    slow = atoi(param);
    stop = atoi(comma+1);
    *comma = ',';

    FILTaddint(filt, sLIMslow, slow, 0);
    FILTaddint(filt, sLIMstop, stop, 0);
    sprintf(buf, "%d", slow);
    SUBSTaddsvar(filt, "P.SLOW", buf);
    sprintf(buf, "%d", slow+stop);
    SUBSTaddsvar(filt, "P.STOP", buf);

    return(1);
}

void
LIMbreidbart(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    int index, numposts, oldposts;
    char *iter;
    char **old;
    int numslow, numstop;
    char *to;
    char ret[256];

    int fd;
    char filename[255];
    FILE *logfile;

    old = LIMdata(lim, name);
    numslow = FILTgetint(filt, sLIMslow, 1);
    numstop = FILTgetint(filt, sLIMstop, 1);
    /* -1 in slow means unlimited */
    if (numslow == -1) {
        *ret = 0;
        return;
    }
    to = FILTgetheader(filt, "Newsgroups:");
    iter = to;

    /* BI = (numto+1)/2 */
    index = 2;
    while (iter = strchr(iter+1, ','))
        index++;
    if (old && old[0])
        oldposts = atoi(old[0]);
    else
        oldposts = 0;
    numposts = index/2;
    sprintf(ret, "%d", numposts+oldposts);
    LIMsetdata(lim, name, ret);
    if (old)
        for (index = 1; index < numperiods(); index++)
            if (old[index])
                oldposts += atoi(old[index]);
    sprintf(ret, "%d", numposts+oldposts);
    SUBSTaddsvar(filt, "P.CUR", ret);

    numposts += oldposts;
    /* -1 in stop means never actually block a post */
    if ((numstop != -1) && (numposts > numslow)) {
        if (numposts > numslow + numstop) {
            ERRORseterror(filt, "User exceeded posting limits.");
            LIMblock(lim, name);
        }
        LIMaddsync(lim, name, LIMgetlog);
    }
}

void
LIMgetlog(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    FILE *logfile;
    int fd, numposts, oldposts, numslow, numstop, index;
#if	defined(DO_FILT_NOTIFY)
    int notifythresh;
#endif
    char **data, *ndata;
    char filename[255];

    index = LIMisblock(lim, name);
    if (index && index != kLIMblockthis) {
        LIMresetdata(lim, name);
        return;
    }
    logfile = FILTgetfile(filt, sMSGlog, 1);
    if (logfile)
        return;
    numslow = FILTgetint(filt, sLIMslow, 1);
    numstop = FILTgetint(filt, sLIMstop, 1);

    data = LIMdata(lim, name);
    ndata = LIMnewdata(lim, name);
    numposts = atoi(ndata);

    if (!data || !data[0])
        oldposts = 0;
    else {
        oldposts = atoi(data[0]);
        for (index = 1; index < numperiods(); index++)
            if (data[index]) {
                oldposts += atoi(data[index]);
                numposts += atoi(data[index]);
            }
    }
    if ((numstop != -1) &&
      (numposts > numslow + numstop)) {
        numposts -= numslow + numstop;
        if (numposts > 10) {
            while ((numposts % 10) == 0)
                numposts /= 10;
            if (numposts < 10) {
                fd = CUPfile(FILTgetuser(filt), 60*60*24*3,
                  filename, sizeof(filename),
                  CONFkeywordvalue(FILTkeylist, "hold.deldir"));
                logfile = fdopen(fd, "w");
                FILTaddfile(filt, sMSGlog, filename, logfile, 0);
            }
        }
    } else {
        fd = CUPfile(FILTgetuser(filt),
          POST_LIMITDELAY*(numposts-numslow),
          filename, sizeof(filename),
          CONFkeywordvalue(FILTkeylist, "hold.dir"));
        logfile = fdopen(fd, "w");
        FILTaddfile(filt, sMSGlog, filename, logfile, 0);
#if	defined(DO_FILT_NOTIFY)
        notifythresh = FILTgetint(filt, sLIMnotify, 1);
        if (!oldposts || (((numposts-numslow) / notifythresh) >
          ((oldposts-numslow) / notifythresh)))
            LIMaddfinish(lim, name, LIMsendnot);
#endif
    }
}

#if	defined(DO_FILT_NOTIFY)
void
LIMsendnot(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    NOTIFYsendmail(filt, CONFkeywordvalue(FILTkeylist, "mail"),
      CONFkeywordvalue(FILTkeylist, "mail.text"));
    FILTdelfile(filt, sMSGlog, 1);
}
#endif
#endif /* DO_FILT_POST */

#if	defined(DO_FILT_CROSSPOST) || defined(DO_FILT_FOLLOWUP)
int
LIMcrosspost(filt, name, param)
    FILT *filt;
    char *name, *param;
{
    if ((!param) || (!*param))
        return(0);
    if (!strcmp(name, "C")) {
        FILTaddint(filt, sLIMcrosspost, atoi(param), 0);
        SUBSTaddsvar(filt, "CP", param);
    } else if (!strcmp(name, "F")) {
        FILTaddint(filt, sLIMfollowup, atoi(param), 0);
        SUBSTaddsvar(filt, "FU", param);
    } else
        return(0);
    return(1);
}

void
LIMcrossblock(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    int index;
    char *iter;
    int numblock;

    /* figure out by name which pair of parameters to use */
    if (!strcmp(name, "C")) {
        iter = FILTgetheader(filt, "Newsgroups:");
        numblock = FILTgetint(filt, sLIMcrosspost, 1);
    } else if (!strcmp(name, "F")) {
        if (!(iter = FILTgetheader(filt, "Followup-To:")))
            iter = FILTgetheader(filt, "Newsgroups:");
        if (FILThasint(filt, sLIMfollowup, 1))
            numblock = FILTgetint(filt, sLIMfollowup, 1);
        else
            numblock = FILTgetint(filt, sLIMcrosspost, 1);
    } else
        return;

    if (numblock == -1)
        return;

    /* figure out number of newsgroups by counting commas */
    index = 1;
    while (iter = strchr(iter+1, ','))
        index++;

    if (index>numblock) {
        switch (*name) {
        case 'C':
            ERRORseterror(filt, "Crossposted to too many groups.");
            break;
        case 'F':
            ERRORseterror(filt, "Followups set to too many groups.");
            break;
        }
        LIMblock(lim, name);
    }
}
#endif

#if	defined(DO_FILT_NOTIFY)
int
LIMnotify(filt, name, param)
    FILT *filt;
    char *name, *param;
{
    if (!param || !(*param))
        return(0);
    FILTaddint(filt, "lim.notify", atoi(param), 0);
    SUBSTaddsvar(filt, "NOT", param);
    return(1);
}
#endif

#if	defined(DO_FILT_GROUP)
/* load exclusive hierarchy list */
int
LIMcrossgroup(filt, name, param)
    FILT *filt;
    char *name, *param;
{
    char *iter;
    char *lparen, *rparen;
    char **groups;
    int index;

    iter = param;
    if (!param || !*param)
        return(0);
    index = 1;
    for (iter = param; iter && *iter; iter++) {
        if (*iter == '(')
            iter = STRfindrparen(iter+1);
        else if (*iter ==',')
            index++;
        else
            return(0);
    }
    if (!iter)
        return(0);
    groups = (char**)malloc(sizeof(char*)*(index+1));
    index = 0;
    for (iter = param; iter && *iter; index++) {
        if (*iter == '(') {
            rparen = STRfindrparen(iter+1);
            *rparen = 0;
            groups[index] = (char*)malloc(strlen(iter));
            strcpy(groups[index], iter+1);
            iter = rparen+1;
        }
        while (*iter == ',')
            iter++;
    }
    groups[index] = 0;
    FILTadddata(filt, kFILTcharpptype, sLIMgroup, (void*)groups,
      (index+1)*sizeof(char*), 0);
    free((void*)groups);
    return(1);
}

/* test whether news posts fall into more than one of some mutually exclusive
 * newsgroup hierarchies
 */
void
LIMgroupblock(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    char **grplist;
    char ***pat;
    char **savpat;
    char *ng[2];
    int nummatch, newmatch;
    int index, index2;
    int bad;
    char *savto;
    char **groups;
    size_t s;

    groups = (char**)FILTgetdata(filt, kFILTcharpptype, sLIMgroup, &s, 1);
    grplist = 0;
    savto = (char*)malloc(strlen(FILTgetheader(filt, "Newsgroups:"))+1);
    strcpy(savto, FILTgetheader(filt, "Newsgroups:"));
    LIMnggetlist(&grplist, savto);
    nummatch = -1;
    bad = 0;
    ng[1] = 0;

    /* fill in the patterns now */
    for (index = 0; groups[index]; index++)
        ;
    index++;
    pat = (char***)malloc(index*sizeof(char**));
    savpat = (char**)malloc(index*sizeof(char*));
    for (index = 0; groups[index]; index++) {
        savpat[index] = (char*)malloc(strlen(groups[index])+1);
        strcpy(savpat[index], groups[index]);
        pat[index] = 0;
        LIMnggetlist(&pat[index], savpat[index]);
    }
    pat[index] = 0;
    savpat[index] = 0;

    /* check for multiple group matches */
    for (index = 0; grplist[index] && !bad; index++) {
        ng[0] = grplist[index];
        newmatch = 0;
        for (index2 = 0; pat[index2] && !newmatch; index2++)
            if (LIMpermmatch(pat[index2], ng))
                newmatch = index2+1;
        if (nummatch == -1)
            nummatch = newmatch;
        else if (nummatch != newmatch)
            bad = 1;
    }

    /* free the patterns */
    for (index = 0; groups[index]; index++) {
        free((void*)savpat[index]);
        free((void*)pat[index]);
    }
    free((void*)savpat);
    free((void*)pat);

    free((void*)savto);
    free((void*)grplist);
    if (bad) {
        ERRORseterror(filt, "Crossposted between mutually exclusive hierarchies.");
        LIMblock(lim, name);
    }
}
#endif

#if	defined(DO_FILT_EMILY)
int
LIMemily(filt, name, param)
    FILT *filt;
    char *name;
    char *param;
{
    int index = 1;
    char *iter = param;
    char *lparen;
    char **emgroups, **emfiles;
    int done = 0;

    if (!param || !*param)
        return(0);
    /* example line: */
    /* (net.*),/usr/news/emily/net,(local.*),/usr/news/emily/local */
    while (!done) {
        if (*iter != '(')
            return 0;
        if (!(iter = STRfindrparen(iter+1)))
            return 0;
        if (*++iter != ',')
            return 0;
        index++;
        if (!(iter = strchr(iter+1, ',')))
            done = 1;
    }
    emgroups = (char**)malloc((index+1)*sizeof(char*));
    emfiles = (char**)malloc((index+1)*sizeof(char*));
    done = 0;
    index = 0;
    iter = param;
    while (!done) {
        lparen = iter;
        iter = STRfindrparen(iter+1);
        *iter = 0;
        emgroups[index] = (char*)malloc(iter-lparen+1);
        strcpy(emgroups[index], lparen+1);
        iter++;
        lparen = ++iter;  /* skip comma */
        if (iter = strchr(iter, ',')) {
            *iter++ = 0;
            emfiles[index] = (char*)malloc(iter-lparen);
        } else {
            emfiles[index] = (char*)malloc(strlen(lparen)+1);
            done = 1;
        }
        strcpy(emfiles[index], lparen);
        index++;
    }
    emgroups[index] = 0;
    emfiles[index] = 0;
    FILTadddata(filt, kFILTcharpptype, sLIMemgroup, (void*)emgroups,
      (index+1)*sizeof(void*), 0);
    free((void*)emgroups);
    FILTadddata(filt, kFILTcharpptype, sLIMemfile, (void*)emfiles,
      (index+1)*sizeof(void*), 0);
    free((void*)emfiles);
    return(1);
}

void
LIMpostnews(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    char *stat;
    char *iter, *lparen;
    char *savng, **nglist, *curng[2];
    char *savpat, **pat;
    char *newng;
    char **emgroups, emfilename[255];
    int *emindex;
    int inverse, index, index2, in, bad;
    size_t s;
    FILE *logfile;
    char err[1024];

    /* example line: */
    /* (net.*)!(local.*) */
    stat = LIMstatdata(lim, name);
    iter = stat;
    /* strategy:
     * Build a list of newsgroups in which you don't have explicit permission
     * to post.  Run these through LIMnewsstuff.emgroups, add an index for
     * any place where you need to see the posting rules.
     * If any of your newsgroups fall into a !() rule, set the error
     * appropriately and block the message. */

    /* get our newsgroup list */
    savng = (char*)malloc(strlen(FILTgetheader(filt, "Newsgroups:"))+1);
    strcpy(savng, FILTgetheader(filt, "Newsgroups:"));
    nglist = 0;
    LIMnggetlist(&nglist, savng);
    curng[1] = 0;

    bad = 0;
    while (iter && *iter) {
        if (*iter == '!') {
            inverse = 1;
            iter++;
        } else
            inverse = 0;
        /* fill in our pattern list */
        lparen = iter;
        iter = STRfindrparen(lparen+1);
        *iter = 0;
        savpat = (char*)malloc(iter-lparen+1);
        strcpy(savpat, lparen+1);
        *iter++ = ')';
        pat = 0;
        LIMnggetlist(&pat, savpat);

        /* index through our newsgroup list checking for hits */
        in = 0;
        for (index = 0; !in && nglist[index]; index++) {
            curng[0] = nglist[index];
            if (LIMpermmatch(pat, curng)) {
                /* found a match for this newsgroup, remove it from the
                 * list */
                in = 1;
                for (index2 = index; nglist[index2]; index2++)
                    nglist[index2] = nglist[index2+1];
            }
        }
        if (in && inverse) {
            if (!bad) {
                bad = 1;
                strcpy(err, "You do not have posting permission in ");
            } else
                strncat(err, ", ", sizeof(err)-strlen(err)-2);
            strncat(err, curng[0], sizeof(err)-strlen(err)-2);
        }

        /* clean up */
        free((void*)pat);
        free((void*)savpat);
    }
    /* Now, we've got a list of newsgroups for which we don't have explicit
     * permissions set.  See if we have a rules file for any of them, if so,
     * prepare to send out mail detailing the rules */
    emindex = 0;
    emgroups = (char**)FILTgetdata(filt, kFILTcharpptype, sLIMemgroup, &s, 1);
    in = -1;
    for (index = 0; nglist[index]; index++) {
        curng[0] = nglist[index];
        for (index2 = 0; emgroups[index2]; index2++) {
            savpat = (char*)malloc(strlen(emgroups[index2])+1);
            strcpy(savpat,emgroups[index2]);
            pat = 0;
            LIMnggetlist(&pat, savpat);
            if (LIMpermmatch(pat, curng)) {
                in++;
                emindex = (int*)realloc(emindex,
                  (in+2)*sizeof(int));
                emindex[in] = index2;
                if (!bad) {
                    bad = 1;
                    strcpy(err, "You do not have posting permission in ");
                } else
                    strncat(err, ", ", sizeof(err)-strlen(err)-2);
                strncat(err, curng[0], sizeof(err)-strlen(err)-2);
            }
            free((void*)savpat);
            free((void*)pat);
        }
    }
    /* We've got to send out rules. */
    if (in >= 0) {
        emindex[in+1] = -1;
        LIMaddfinish(lim, name, LIMsendrules);
        in = CUPfile(FILTgetuser(filt), 0, emfilename, sizeof(emfilename),
          _PATH_TMP);
        logfile = fdopen(in, "w");
        FILTaddfile(filt, sMSGlog, emfilename, logfile, 0);
        FILTadddata(filt, kFILTemtype, sLIMemindex, (void*)emindex,
          (in+2)*sizeof(int), 0);
        free((void*)emindex);
    }

    /* clean up */
    free((void*)nglist);
    free((void*)savng);


    if (bad) {
        strcat(err, ".");
        ERRORseterror(filt, err);
        LIMblock(lim, name);
    }
}

void
LIMsendrules(filt, lim, name)
    FILT *filt;
    LIM *lim;
    char *name;
{
    char buf[1024];
    FILE *emtext;
    char *text;
    char cmdline[255];
    long start, end;
    int index, ifile, index2;
    char *emfilename;
    char **emfiles;
    int *emindex;
    size_t s;

    emfilename = FILTgetfname(filt, sMSGlog, 1);
    emindex = FILTgetdata(filt, kFILTemtype, sLIMemindex, &s, 1);
    emfiles = FILTgetdata(filt, kFILTcharpptype, sLIMemfile, &s, 1);
    CUPsetfilename(filt, emfilename);
    FILTdelfile(filt, sMSGlog, 1);

    for (index = 0; emindex[index] >= 0; index++) {
        ifile = emindex[index];
        /* skip any rules we've sent out before. */
        for (index2 = 0; index2 < index; index2++)
            if (emindex[index2] == ifile)
                continue;
        if (!(emtext = fopen(emfiles[ifile], "r")))
            continue;
        fgets(cmdline, sizeof(cmdline), emtext);
        cmdline[strlen(cmdline)-1] = 0;
        start = ftell(emtext);
        fseek(emtext, 0, SEEK_END);
        end = ftell(emtext);
        text = (char*)malloc(end-start+1);
        fseek(emtext, start, SEEK_SET);
        fread(text, (end-start), 1, emtext);
        fclose(emtext);
        text[end-start] = 0;
        NOTIFYsendmail(filt, cmdline, text);
        free((void*)text);
    }
    FILTdeldata(filt, kFILTemtype, sLIMemindex, 1);
    unlink(CUPgetfilename(filt));
}
#endif
