/* authenticate a user using a strange method */
/* Revision history
 * 19970915 - Aidan Cully
 *     Added revision history
 *     Now can attempt several kinds of authentication on the same line
 */
#include <pwd.h>
#include <stdio.h>
#include <string.h>

#include "passwd.h"
#include "authfuncs.h"

#define kPERMWantsNoPass 0x1
#define kPERMWantsPass 0x2
#define kPERMWantsAll -1

typedef struct _PERMauthtype {
    char *name;
    int kind; /* when to invoke this function */
    PERMinitfunc init;
    PERMauthfunc func;
} PERMauthtype;

PERMauthtype PERMauth[] =
 {
#if	defined(DO_AUTH_PW)
   { "PW",	kPERMWantsPass, 	0, PERMpwfile },
#endif
#if	defined(DO_AUTH_DB)
   { "DB",	kPERMWantsPass, 	0, PERMpwfile },
#endif
#if	defined(DO_AUTH_IPNAME)
   { "IP-NAME",	kPERMWantsNoPass,	0, IPNAMEresolveuser },
#endif
#if	defined(DO_AUTH_IDENT)
   { "IDENT",	kPERMWantsNoPass,	IDENTinitencrypt, IDENTresolveuser },
#endif
#if	defined(DO_AUTH_SET)
   { "SET",	kPERMWantsNoPass,	0, PERMsetuser },
#endif
#if	defined(DO_AUTH_TAG)
   { "TAG",	kPERMWantsAll,  	0, PERMnametag },
#endif
#if	defined(DO_AUTH_IPUSER)
   { "IP-USER",	kPERMWantsNoPass,	0, PERMipuser },
#endif
   { 0, 0, 0, 0 }
 };

void
PERMinitpasswd()
{
    int index;

    for (index = 0; index<sizeof(PERMauth)/sizeof(PERMauthtype)-1; index++)
        if (PERMauth[index].init)
            PERMauth[index].init(PERMauth[index].name);
}

int
PERMcheckpasswd(user, pass, param, defdom, realuser, len)
    char *user, *pass, *param, *defdom, *realuser;
    size_t len;
{
    char *lparen, *rparen, *name, *iter, *domain;
    int found, index, ret, done;

    ret = 0;
    done = 0;
    rparen = param;
    while (!ret && !done) {
        if (*rparen == ',') {
            rparen++;
            continue;
        } else if (!*rparen) {
            done = 1;
            break;
        }
        name = rparen;
        lparen = name;
        rparen = 0;
        found = 0;
        domain = 0;
        iter = 0;
        /* get the identifier, flag if we're at the end of the line */
        while (*lparen && !found) {
            if ((*lparen == ',') || (*lparen == '@')) {
                *lparen = 0;
                found = 1;
                if (*lparen == '@') {
                    domain = lparen+1;
                    if (iter = strchr(domain, ','))
                        *iter = 0;
                    else
                        done = 1;
                }
            } else if (*lparen == '(') {
                found = 1;
                rparen = lparen+1;
                /* allow for nesting parenthesis */
                while (found && *rparen) {
                    if (*rparen == ')')
                        found--;
                    else if (*rparen == '(')
                        found++;
                    if (found)
                        rparen++;
                }
                if (found)
                    return(0);
                found = 1;
                *lparen = 0;
                *rparen = 0;
                if (*(rparen+1) == '@') {
                    domain = rparen+2;
                    if (iter = strchr(domain, ','))
                        *iter = 0;
                    else
                        done = 1;
                }
            } else if (!*lparen)
                done = found = 1;
            else
                lparen++;
        }
        /* malformed line */
        if (!found)
            return(0);
        found = 0;
        /* get the authenticator */
        index = 0;
        while (!found && (index<sizeof(PERMauth)/sizeof(PERMauthtype)-1)) {
            if (!strcmp(PERMauth[index].name, name))
                found = 1;
            else
                index++;
        }
        /* attempt authentication if the function exists and is compatible
         * with the parameters given to this function */
        if (found &&
          ((pass && (PERMauth[index].kind & kPERMWantsPass)) ||
          (!pass && (PERMauth[index].kind & kPERMWantsNoPass))))
            ret = PERMauth[index].func(name, user, pass, rparen?lparen+1:0,
              realuser, len);

        /* find the end of this identifier and continue */
        if (domain)
            rparen = iter;
        else if (!rparen)
            rparen = lparen;
        /* skip the identifier's delimiter */
        rparen++;
    }
    if ((ret != kPERMUserBad) && (ret != kPERMUserOK) &&
      (ret != kPERMUserWithDomainOK))
        ret = 0;

    if ((ret == kPERMUserOK) || (ret == kPERMUserWithDomainOK)) {
        if (defdom && *defdom && !domain)
            domain = defdom;
        if ((ret != kPERMUserWithDomainOK) && domain) {
            strncat(realuser, "@", len);
            strncat(realuser, domain, len);
            ret = kPERMUserWithDomainOK;
        }
    }
    return ret;
}
