#ifndef __LIMITFILE_H
#define __LIMITFILE_H

#include <sys/types.h>
#include "filter.h"

typedef struct {} LIM;

typedef int (*LIMparamfunc)(FILT*, char*, char*);
typedef void (*LIMdetectfunc)(FILT*, LIM*, char*);
typedef void (*LIMsyncfunc)(FILT*, LIM*, char*);
typedef void (*LIMfinishfunc)(FILT*, LIM*, char*);

typedef struct _LIMent {
    char **param;
    LIMparamfunc pf;
    LIMdetectfunc df;
} LIMent;

#define kLIMnotblocked	0
#define kLIMblockthis	1
#define kLIMblockelse	2

extern void LIMsetparams(LIMent*, int);
extern void LIMtrackpost(FILT*);
extern void LIMfinish(FILT*);
extern int LIMshouldsend(FILT*);

/* what filters are installed? */
extern char *LIMfirstfilt();
extern char *LIMnextfilt(char*); /* prev */

/* data accessor functions.  Determine what data goes into the persistant
 * database */

/* dynamic data */
extern char **LIMdata(LIM*, char*); /* state, name */
extern char *LIMnewdata(LIM*, char*); /* state, name */
extern void LIMsetdata(LIM*, char*, char*); /* state, name, new */
extern void LIMcleardata(LIM*, char*); /* state, name */
extern void LIMresetdata(LIM*, char*); /* state, name */

/* static data */
extern char *LIMstatdata(LIM*, char*); /* state, name */
extern char *LIMnewstatdata(LIM*, char*); /* state, name */
extern void LIMsetstatdata(LIM*, char*, char*); /* state, name, newstr */
extern void LIMclearstatdata(LIM*, char*); /* state, name */
extern void LIMresetstatdata(LIM*, char*); /* state, name */


/* post behavior */
extern void LIMblock(LIM*, char*); /* state, name */
extern void LIMclearblock(LIM*, char*); /* state, name */
extern int LIMisblock(LIM*, char*); /* state, name */

/* Register postprocessors.  These take over after the post has been logged
 * or whatnot, and finish sending out notification or what have you. */
extern void LIMaddfinish(LIM*, char*, LIMfinishfunc);
extern void LIMresetfinish(LIM*, char*);
extern void LIMaddsync(LIM*, char*, LIMsyncfunc);
extern void LIMresetsync(LIM*, char*);

#endif
