#include <string.h>
#include "configdata.h"
#include "misc.h"
#include "macros.h"

/*
** This is a utility function for getting to the meat of a line of text.
** It returns a pointer to the first non-blank character in the line,
** or null if it is a comment or empty line.
*/
char *
MISCparseline(buf)
    char *buf;
{
    char *hash;

    /* Skip past whitespace and empty lines */
    while (buf[0]&&((buf[0]==' ')||(buf[0]=='\t')))
        buf++;
    hash = strchr(buf, '#');
    if (hash)
        *hash = 0;
    if (!buf[0]||(buf[0]=='\n'))
        return(0);
    return(buf);
}

/* STRgetparam */
/* Search a string of the form F1(args),F2(args) for item F1, return the
 * appropriate arguments.
 * Returns 1 if it found the proper item, 0 if not */
int
STRgetparam(prefix, str, ret, len)
    char *prefix, *str, *ret;
    size_t len;
{
    char *iter, *lparen, *rparen;
    int found, nest;

    iter = str;
    found = 0;
    while (!found && *iter) {
        if (*iter == ',') {
            iter++;
            continue;
        }
        if (!(lparen = strchr(iter, '('))) {
            /* check for parameterless field */
            if (rparen = strchr(iter, ','))
                *rparen = 0;
        } else {
            if (!(rparen = STRfindrparen(lparen+1)))
                return 0;
            *lparen = 0;
        }
        if (!strcmp(prefix, iter)) {
            /* got a match! store the value */
            *rparen = 0;
            strncpy(ret, lparen+1, len);
            ret[len] = 0;
            found = 1;
        } else
            /* skip past stuff beloning to this field */
            iter = rparen+1;
        if (lparen) {
            *lparen = '(';
            *rparen = ')';
        } else if (rparen)
            *rparen = ',';
    }
    if (!found)
        *ret = 0;
    return (found);
}

char *
STRfindrparen(str)
    char *str;
{
    int in = 1;
    char *iter;

    iter = str;
    while (*iter && in) {
        if (*iter == '(')
            in++;
        if (*iter == ')')
            in--;
        iter++;
    }
    if (in)
        return 0;
    return (iter-1);
}

/* functions that used to be in nnrpd, but had to move here so that I could
 * use them with rnews */

/*
**  Parse a string into a NULL-terminated array of words; return number
**  of words.  If argvp isn't NULL, it and what it points to will be
**  DISPOSE'd.
*/
int
LIMargify(line, argvp)
    char                *line;
    char                ***argvp;
{
    register char       **argv;
    register char       *p;
    register int        i;

    if (*argvp != NULL) {
        DISPOSE(*argvp[0]);
        DISPOSE(*argvp);
    }

    /*  Copy the line, which we will split up. */
    while (ISWHITE(*line))
        line++;
    i = strlen(line);
    p = NEW(char, i + 1);
    (void)strcpy(p, line);

    /* Allocate worst-case amount of space. */
    for (*argvp = argv = NEW(char*, i + 2); *p; ) {
        /* Mark start of this word, find its end. */
        for (*argv++ = p; *p && !ISWHITE(*p); )
            p++;
        if (*p == '\0')
            break;

        /* Nip off word, skip whitespace. */
        for (*p++ = '\0'; ISWHITE(*p); )
            p++;
    }
    *argv = NULL;
    return argv - *argvp;
}

/*
**  Match a list of newsgroup specifiers against a list of newsgroups.
**  func is called to see if there is a match.
*/
BOOL
LIMpermmatch(Pats, list)
    char                **Pats;
    char                **list;
{
    register int        i;
    register char       *p;
    register BOOL	match;

    if (Pats[0] == NULL)
        return TRUE;

    for ( ; *list; list++)
        for (i = 0; (p = Pats[i]) != NULL; i++) {
            if (p[0] == '!') {
                if (wildmat(*list, ++p))
                    match = FALSE;
            }
            else if (wildmat(*list, p))
                match = TRUE;
        }

    return match;
}

/*
**  Parse a newsgroups line, return TRUE if there were any.
*/
BOOL
LIMnggetlist(argvp, list)
    char                ***argvp;
    char                *list;
{
    register char       *p;

    for (p = list; *p; p++)
        if (*p == ',')
            *p = ' ';

    return LIMargify(list, argvp) != 0;
}
