#include <stdio.h>
#include "parse.h"
#include "filtpaths.h"
#include "postincr.h"

int doquit(char*, char*);
int douser(char*, char*);
int dodata(char*, char*);

typedef struct _CMDstruct {
    char *name;
    int (*func)(char*, char*);
} CMDstruct;

int done = 0;
char user[255];
CMDstruct cmds[] =
 { {"QUIT", doquit},
   {"USER", douser},
   {"PRINT", dodata},
   {"SET", dodata},
   {"DELETE", dodata} };

void
doline(line, cmdlist, numcmds)
    char *line;
    CMDstruct *cmdlist;
    int numcmds;
{
    char tmp[255];
    char *iter;
    int index, ret;

    iter = PARSEgettoken(line, tmp);
    for (index = 0; index<numcmds; index++) {
        if (!strcasecmp(cmdlist[index].name, tmp)) {
            ret = cmdlist[index].func(cmdlist[index].name, iter);
            if (!ret)
                printf("Command not understood.\n");
            return;
        }
    }
    printf("Unknown command.\n");
}

int
main()
{
    char line[255];

    *user = 0;
    while (!done) {
        printf("Ready.\n");
        if (!fgets(line, sizeof(line), stdin))
            done = 1;
        else {
            line[strlen(line)-1] = 0;
            doline(line, cmds, sizeof(cmds)/sizeof(CMDstruct));
        }
    }
}

int
doquit(name, args)
    char *name;
    char *args;
{
    done = 1;
    return (1);
}

int
douser(name, args)
    char *name, *args;
{
    if (!args || !*args)
        return (0);
    args = PARSEgettoken(args, user);
    if (args && *args) {
        *user = 0;
        return (0);
    }
    return (1);
}

int
dodata(name, args)
    char *name, *args;
{
    char modestr[255];
    int mode;
    char **tm;

    if (!strcmp(name, "SET"))
        mode = 4;
    else if (!strcmp(name, "PRINT"))
        mode = 8;
    else if (!strcmp(name, "DELETE"))
        mode = 12;
    else
        return (0);
    if (!*user) {
        printf("User not set.  Use USER command.\n");
        return(1);
    }
    args = PARSEgettoken(args, modestr);
    if (!args)
        return (0);
    if (!strcasecmp(modestr, "STATIC"))
        mode |= 1;
    else if (!strcasecmp(modestr, "DYNAMIC")) {
        mode |= 2;
        tm = (char**)malloc((numperiods()+1)*sizeof(char*));
    } else
        return (0);
    opendb();
    if (mode & 1) {
        if ((mode & 12) == 4) {
            setstatic(user, args);
            printf("%s: %s set.\n", user, args);
        } else if ((mode & 12) == 8) {
            getstatic(user, modestr, sizeof(modestr));
            printf("%s: %s\n", user, modestr);
        } else if ((mode & 12) == 12) {
            setstatic(user, "");
            printf("%s static deleted.\n", user);
        }
    } else {
        if ((mode & 12) == 4) {
            setposts(user, args);
            printf("%s: %s set.\n", user, args);
        } else if ((mode & 12) == 8) {
            numposts(user, tm);
            for (mode = 0; mode < numperiods(); mode++) {
                printf("%s[%d]: %s\n", user, mode, tm[mode]);
                free((void*)tm[mode]);
            }
            free((void*)tm);
        } else if ((mode & 12) == 12) {
            setposts(user, "");
            printf("%s current deleted.\n", user);
        }
    }
    closedb();
    return(1);
}
