#include <string.h>
#include <malloc.h>
#include "subst.h"
#include "substfuncs.h"

SUBSTinfo substfuncs[] =
  { {"U", 0, SUBSTusername},
    {"F", 0, SUBSTfilename},
    {"E", 0, SUBSTgeterror},
    {"T", SUBSTfilelen, SUBSTfiletext} };

FILT *SUBSTglobvar = 0;

/*
** adds a variable to the subst in the FILT*
*/
void
SUBSTaddsvar(filt, name, arg)
    FILT *filt;
    char *name, *arg;
{
    size_t s;
    FILT *sfilt;

    sfilt = FILTgetfilt(filt, "subst.data");
    if (!sfilt) {
        sfilt = FILTalloc();
        FILTaddfilt(filt, "subst.data", sfilt);
    }
    FILTaddstring(sfilt, name, arg);
}

/*
** changes an identifier to a string
*/
void
SUBSTgetvalue(arg, ret, len, parm)
    char *arg, *ret;
    size_t len;
    FILT *parm;
{
    int i, numfuncs;
    FILT *sfilt;
    size_t s;
    char *hold;

    numfuncs = sizeof(substfuncs)/sizeof(SUBSTinfo);
    i=0;
    while ((i<numfuncs)&&strcmp(substfuncs[i].name, arg))
        i++;
    if (i==numfuncs) {
        sfilt = FILTgetfilt(parm, "subst.data");
        if (!sfilt)
            hold = 0;
        else
            hold = FILTgetstring(sfilt, arg);
        if (!hold) {
            if (SUBSTglobvar)
                hold = FILTgetstring(SUBSTglobvar, arg);
        }
        if (!hold)
            *ret = 0;
        else {
            strncpy(ret, hold, len);
            ret[len] = 0;
        }
    } else
        substfuncs[i].func(arg, ret, len, parm);
}

/*
** gets the length of the string an identifier would yield
*/
size_t
SUBSTgetlength(arg, parm)
    char *arg;
    FILT *parm;
{
    int i, numfuncs;
    char buf[256];
    FILT *sfilt;
    char *hold;
    size_t s;

    numfuncs = sizeof(substfuncs)/sizeof(SUBSTinfo);
    i=0;
    while ((i<numfuncs)&&strcmp(substfuncs[i].name, arg))
        i++;
    if (i==numfuncs) {
        sfilt = FILTgetfilt(parm, "subst.data");
        if (!sfilt) {
             hold = 0;
        } else
            hold = FILTgetstring(sfilt, arg);
        if (!hold) {
            if (SUBSTglobvar)
                hold = FILTgetstring(SUBSTglobvar, arg);
        }
        if (!hold)
            return(0);
        return(strlen(hold));
    }
    if (substfuncs[i].len)
        return(substfuncs[i].len(arg, parm));
    SUBSTgetvalue(arg, buf, sizeof(buf), parm);
    return(strlen(buf));
}

/*
** Takes a bunch of text and performs neat substitution on it.
*/
void
SUBSTchangetext(rtext, ret, len, parm)
    char *rtext, *ret;
    size_t len;
    FILT *parm;
{
    char *close;
    char *text;

    text=(char*)malloc(sizeof(char)*(strlen(rtext)+1));
    if (!text) {
        *ret = 0;
        return;
    }
    strcpy(text, rtext);
    rtext = text;
    while (*text&&(len>0)) {
        if ((*text=='<')&&(*(text-1)!='\\')) {
            if ((*++text)=='%') {
                close=strchr(text, '>');
                text++;
                *close=0;
                SUBSTgetvalue(text, ret, len, parm);
                len-= strlen(ret);
                ret+= strlen(ret);
                text=close+1;
            }
        }
        len--;
        *ret++=*text++;
    }
    if (!len)
        ret--;
    *ret=0;
    free(rtext);
}

/*
** Takes a bunch of text and figures out how long changetext would be
*/
size_t
SUBSTlengthtext(rtext, parm)
    char *rtext;
    FILT *parm;
{
    char *close;
    char *text;
    size_t len;

    text=(char*)malloc(sizeof(char)*(strlen(rtext)+1));
    if (!text)
        return(0);
    strcpy(text, rtext);
    rtext = text;
    len = 0;
    while (*text) {
        if ((*text=='<')&&(*(text-1)!='\\')) {
            if ((*++text)=='%') {
                close=strchr(text, '>');
                text++;
                *close=0;
                len += SUBSTgetlength(text, parm);
                text=close+1;
            }
        }
        len++;
        text++;
    }
    free(rtext);
    len++;
    return(len);
}
