#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include "clibrary.h"
#include "configdata.h"
#include "paths.h"
#include "track.h"
#include "limitfile.h"
#include "error.h"
#include "misc.h"
#include "filter.h"
#include "catchup.h"
#include "proto.h"
#include "subst.h"

#if	defined(DO_FILT_POST)
static char *LIMpostarr[] = {"P", "Post", 0};
#endif
#if	defined(DO_FILT_CROSSPOST)
static char *LIMcrosspostarr[] = {"C", "Crosspost", 0};
#endif
#if	defined(DO_FILT_FOLLOWUP)
static char *LIMfollowuparr[] = {"F", "Follow-up", 0};
#endif
#if	defined(DO_FILT_NOTIFY)
static char *LIMnotifyarr[] = {"N", "Notify", 0};
static char *LIMnotifymsgarr[] = {"NM", "Not.msg", "Notify.msg", 0};
#endif
#if	defined(DO_FILT_GROUP)
static char *LIMgrouparr[] = {"G", "Group", "Groups", "CrossGroup", "CrossGroups", 0};
#endif
#if	defined(DO_FILT_EMILY)
static char *LIMemilyarr[] = {"E", "Emily", 0};
#endif
static char *LIMdbarr[] = {"DB", 0};
static char *LIMerrmsgarr[] = {"ERR", "ErrMsg", "ErrorMessage", 0};

LIMload LIMloadlist[] =
 {
#if	defined(DO_FILT_POST)
   { LIMpostarr,	LIMpost},
#endif
#if	defined(DO_FILT_CROSSPOST)
   { LIMcrosspostarr,	LIMcrosspost},
#endif
#if	defined(DO_FILT_FOLLOWUP)
   { LIMfollowuparr,	LIMcrosspost},
#endif
#if	defined(DO_FILT_NOTIFY)
   { LIMnotifyarr,	LIMnotify},
   { LIMnotifymsgarr,	LIMnotifymsg},
#endif
#if	defined(DO_FILT_GROUP)
   { LIMgrouparr,	LIMcrossgroup},
#endif
#if	defined(DO_FILT_EMILY)
   { LIMemilyarr,	LIMemily},
#endif
   { LIMdbarr,		LIMdb },
   { LIMerrmsgarr,	LIMerrmsg },
   { 0, 0 }
 };

LIMlimfunc LIMlimlist[] =
 {
#if	defined(DO_FILT_POST)
   LIMbreidbart,
#endif
#if	defined(DO_FILT_CROSSPOST) || defined(DO_FILT_FOLLOWUP)
   LIMcrossblock,
#endif
#if	defined(DO_FILT_GROUP)
   LIMgroupblock,
#endif
#if	defined(DO_FILT_EMILY)
   LIMpostnews,
#endif
   LIMrundb,
   0
 };

void
PROTOdelcharpp(type, data, len)
    int type;
    void *data;
    size_t len;
{
    char **real = (char**)data;
    int index;

    for (index = 0; real[index]; index++)
        free((void*)real[index]);
}

void
TRACKinit()
{
    FILTadddel(kFILTcharpptype, PROTOdelcharpp);
}

#if	defined(DO_FILT_POST)
int
LIMpost(filt, name, params)
    FILT *filt;
    char *name;
    LIMconfpart *params;
{
    char *comma;
    char tok[80], tok2[80];
    int line, val;

    if (!params[0].lines)
        return(0);
    if (params[1].lines)
        return(0);
    line = 0;
    comma = 0;
    if (!(comma = LIMgetparttok(comma, tok, &line, &params[0])))
        return(0);
    if (!strcasecmp(tok, "unlimited")) {
        if (comma = LIMgetparttok(comma, tok, &line, &params[0]))
            return(0);
        FILTaddint(filt, sLIMslow, -1);
        return(1);
    }
    val = atoi(tok);
    if (comma = LIMgetparttok(comma, tok2, &line, &params[0])) {
        FILTaddint(filt, sLIMslow, val);
        FILTaddint(filt, sLIMstop, atoi(tok2));
        val += atoi(tok2);
        sprintf(tok2, "%d", val);
    } else {
        FILTaddint(filt, sLIMslow, val);
        FILTaddint(filt, sLIMstop, 0);
    }
    return(1);
}

void
LIMbreidbart(globfilt, filt, lim)
    FILT *globfilt, *filt;
    LIM *lim;
{
    int index, numposts, oldposts;
    char *iter;
    char **old;
    int numslow, numstop;
    char *to;
    char ret[256];

    int fd;
    char filename[255];
    FILE *logfile;

    if (!LIMparmloaded(filt, "P"))
        return;
    old = LIMdata(lim, "P");
    numslow = FILTgetint(filt, sLIMslow);
    numstop = FILTgetint(filt, sLIMstop);
    /* -1 in slow means unlimited */
    if (numslow == -1) {
        *ret = 0;
        return;
    }
    to = FILTgetheader(filt, "Newsgroups:");
    iter = to;

    /* BI = (numto+1)/2 */
    index = 2;
    while (iter = strchr(iter+1, ','))
        index++;
    if (old && old[0])
        oldposts = atoi(old[0]);
    else
        oldposts = 0;
    numposts = index/2;
    sprintf(ret, "%d", numposts+oldposts);
    LIMsetdata(lim, "P", ret);
    sprintf(ret, "%d", numslow);
    SUBSTaddsvar(filt, "P.SLOW", ret);
    sprintf(ret, "%d", numslow+numstop);
    SUBSTaddsvar(filt, "P.STOP", ret);

    if (old)
        for (index = 1; index < numperiods(); index++)
            if (old[index])
                oldposts += atoi(old[index]);
    numposts += oldposts;
    sprintf(ret, "%d", numposts);
    SUBSTaddsvar(filt, "P.CUR", ret);

    /* -1 in stop means never actually block a post */
    if ((numstop != -1) && (numposts > numslow)) {
        if (numposts > numslow + numstop) {
            if (LIMgeterrmsg(filt, "P"))
                ERRORseterror(globfilt, LIMgeterrmsg(filt, "P"));
            else
                ERRORseterror(globfilt, "User exceeded posting limits.");
            LIMblock(lim, "P");
        }
        LIMaddsync(lim, "P", LIMgetlog);
    }
}

void
LIMgetlog(filt, locfilt, lim)
    FILT *filt, *locfilt;
    LIM *lim;
{
    FILE *logfile;
    int fd, numposts, oldposts, numslow, numstop, index;
#if	defined(DO_FILT_NOTIFY)
    int notifythresh;
#endif
    char **data, *ndata;
    char filename[255];

    index = LIMisblock(lim, "P");
    if (index && index != kLIMblockthis) {
        LIMresetdata(lim, "P");
        return;
    }
    logfile = FILTgetfile(filt, sMSGlog);
    if (logfile)
        return;
    numslow = FILTgetint(locfilt, sLIMslow);
    numstop = FILTgetint(locfilt, sLIMstop);

    data = LIMdata(lim, "P");
    ndata = LIMnewdata(lim, "P");
    numposts = atoi(ndata);

    if (!data || !data[0])
        oldposts = 0;
    else {
        oldposts = atoi(data[0]);
        for (index = 1; index < numperiods(); index++)
            if (data[index]) {
                oldposts += atoi(data[index]);
                numposts += atoi(data[index]);
            }
    }
    if ((numstop != -1) &&
      (numposts > numslow + numstop)) {
        numposts -= numslow + numstop;
        if (numposts > 10) {
            while ((numposts % 10) == 0)
                numposts /= 10;
            if (numposts < 10) {
                fd = CUPfile(FILTgetuser(filt), 60*60*24*3,
                  filename, sizeof(filename),
                  CONFkeywordvalue(FILTkeylist, "hold.deldir"));
                logfile = fdopen(fd, "w");
                FILTaddfile(filt, sMSGlog, filename, logfile);
            }
        }
    } else {
        fd = CUPfile(FILTgetuser(filt),
          POST_LIMITDELAY*numposts-numslow,
          filename, sizeof(filename),
          CONFkeywordvalue(FILTkeylist, "hold.dir"));
        logfile = fdopen(fd, "w");
        FILTaddfile(filt, sMSGlog, filename, logfile);
#if	defined(DO_FILT_NOTIFY)
        notifythresh = FILTgetint(locfilt, sLIMnotify);
        if (!oldposts || ((numposts / notifythresh) >
          (oldposts / notifythresh))) {
            /* TODO: Make this work. */
            LIMaddfinish(lim, "P", LIMsendnot);
        }
#endif
    }
}

#if	defined(DO_FILT_NOTIFY)
void
LIMsendnot(filt, locfilt, data)
    FILT *filt, *locfilt;
    LIM *data;
{
    char *filename;
    char tok[10];

    filename = FILTgetstring(locfilt, sLIMnotfile);
    CUPsetfilename(locfilt, FILTgetfname(filt, sMSGlog));
    sprintf(tok, "%d", FILTgetint(locfilt, sLIMnotify));
    SUBSTaddsvar(locfilt, "NOT", tok);
    LIMsendmessage(locfilt, filename);
    FILTdelfile(filt, sMSGlog);
}
#endif /* DO_FILT_NOTIFY */
#endif /* DO_FILT_POST */

#if	defined(DO_FILT_CROSSPOST) || defined(DO_FILT_FOLLOWUP)
int
LIMcrosspost(filt, name, params)
    FILT *filt;
    char *name;
    LIMconfpart *params;
{
    char tok[80];
    char *cheese;
    int line;

    if (!params || !params[0].lines || params[1].lines)
        return(0);
    cheese = 0;
    line = 0;
    if (!(cheese = LIMgetparttok(cheese, tok, &line, &params[0])))
        return(0);
#if	defined(DO_FILT_CROSSPOST)
    if (!strcmp(name, "C")) {
        FILTaddint(filt, sLIMcrosspost, atoi(tok));
        SUBSTaddsvar(filt, "CP", tok);
    }
#if	defined(DO_FILT_FOLLOWUP)
    else
#endif
#endif
#if	defined(DO_FILT_FOLLOWUP)
    if (!strcmp(name, "F")) {
        FILTaddint(filt, sLIMfollowup, atoi(tok));
        SUBSTaddsvar(filt, "FU", tok);
    }
#endif
    else
        return(0);
    if (cheese = LIMgetparttok(cheese, tok, &line, &params[0]))
        return(0);
    return(1);
}

int
LIMcrossblockreal(filt, lim, iter, numblock)
    FILT *filt;
    LIM *lim;
    char *iter;
    int numblock;
{
    int index;

    if (numblock == -1)
        return(0);

    /* figure out number of newsgroups by counting commas */
    index = 1;
    while (iter = strchr(iter+1, ','))
        index++;

    if (index>numblock)
        return(1);
    return(0);
}

void
LIMcrossblock(globfilt, filt, lim)
    FILT *globfilt, *filt;
    LIM *lim;
{
    int index;
    char *iter;
    int numblock;

    /* figure out by name which pair of parameters to use */
#if	defined(DO_FILT_CROSSPOST)
    if (LIMparmloaded(filt, "C")) {
        iter = FILTgetheader(filt, "Newsgroups:");
        numblock = FILTgetint(filt, sLIMcrosspost);
        if (LIMcrossblockreal(filt, lim, iter, numblock)) {
            if (LIMgeterrmsg(filt, "C"))
                ERRORseterror(globfilt, LIMgeterrmsg(filt, "C"));
            else
                ERRORseterror(globfilt, "Crossposted to too many groups.");
            LIMblock(lim, "C");
            return;
        }
    }
#endif
#if	defined(DO_FILT_FOLLOWUP)
    if (LIMparmloaded(filt, "F")) {
        if (!(iter = FILTgetheader(filt, "Followup-To:")))
            iter = FILTgetheader(filt, "Newsgroups:");
        if (FILThasint(filt, sLIMfollowup))
            numblock = FILTgetint(filt, sLIMfollowup);
        else
            numblock = FILTgetint(filt, sLIMcrosspost);
        if (LIMcrossblockreal(filt, lim, iter, numblock)) {
            if (LIMgeterrmsg(filt, "F"))
                ERRORseterror(globfilt, LIMgeterrmsg(filt, "F"));
            else
                ERRORseterror(globfilt, "Followups to too many groups.");
            LIMblock(lim, "F");
            return;
        }
    }
#endif
}
#endif

#if	defined(DO_FILT_NOTIFY)
int
LIMnotify(filt, name, params)
    FILT *filt;
    char *name;
    LIMconfpart *params;
{
    char *iter, tok[80];
    int line;

    if (!params || !(params[0].lines) || params[1].lines)
        return(0);
    iter = 0;
    line = 0;
    iter = LIMgetparttok(iter, tok, &line, &params[0]);
    if (!iter)
        return(0);
    FILTaddint(filt, "lim.notify", atoi(tok));
    if (iter = LIMgetparttok(iter, tok, &line, &params[0]))
        return(0);
    return(1);
}

int
LIMnotifymsg(filt, name, params)
    FILT *filt;
    char *name;
    LIMconfpart *params;
{
    char *iter, tok[80];
    int line;

    if (!params || !(params[0].lines) || params[1].lines)
        return(0);
    iter = 0;
    line = 0;
    iter = LIMgetparttok(iter, tok, &line, &params[0]);
    if (!iter)
        return(0);
    FILTaddstring(filt, sLIMnotfile, tok);
    if (iter = LIMgetparttok(iter, tok, &line, &params[0]))
        return(0);
    return(1);
}
#endif

#if	defined(DO_FILT_GROUP)
/* load exclusive hierarchy list */
int
LIMcrossgroup(filt, name, param)
    FILT *filt;
    char *name;
    LIMconfpart *param;
{
    char *iter, *last, tok[80];
    char *lparen, *rparen;
    char **groups;
    int index, line;

    if (!param || !param[0].lines)
        return(0);
    index = 1;
    for (line = 0; param[0].lines[line]; line++) {
        for (iter = param[0].lines[line]; iter && *iter; iter++) {
            if (*iter == '(') {
                iter = STRfindrparen(iter+1);
                index++;
            } else {
		iter = PARSEgettoken(iter, tok);
                index++;
            }
        }
        index++;
    }
    groups = (char**)malloc(sizeof(char*)*(index+1));
    index = 0;
    for (line = 0; param[0].lines[line]; line++) {
        for (iter = param[0].lines[line]; iter && *iter; index++) {
            if (*iter == '(') {
                rparen = STRfindrparen(iter+1);
                *rparen = 0;
                groups[index] = (char*)malloc(strlen(iter));
                strcpy(groups[index], iter+1);
                iter = rparen+1;
            } else {
                iter = PARSEgettoken(iter, tok);
                groups[index] = (char*)malloc(strlen(tok)+1);
                strcpy(groups[index], tok);
            }
        }
    }
    groups[index] = 0;
    FILTadddata(filt, kFILTcharpptype, sLIMgroup, (void*)groups,
      (index+1)*sizeof(char*));
    free((void*)groups);
    return(1);
}

/* test whether news posts fall into more than one of some mutually exclusive
 * newsgroup hierarchies
 */
void
LIMgroupblock(globfilt, filt, lim)
    FILT *globfilt, *filt;
    LIM *lim;
{
    char **grplist;
    char ***pat;
    char **savpat;
    char *ng[2];
    int nummatch, newmatch;
    int index, index2;
    int bad;
    char *savto;
    char **groups;
    size_t s;

    if (!LIMparmloaded(filt, "G"))
        return;
    groups = (char**)FILTgetdata(filt, kFILTcharpptype, sLIMgroup, &s);
    grplist = 0;
    savto = (char*)malloc(strlen(FILTgetheader(filt, "Newsgroups:"))+1);
    strcpy(savto, FILTgetheader(filt, "Newsgroups:"));
    LIMnggetlist(&grplist, savto);
    nummatch = -1;
    bad = 0;
    ng[1] = 0;

    /* fill in the patterns now */
    for (index = 0; groups[index]; index++)
        ;
    index++;
    pat = (char***)malloc(index*sizeof(char**));
    savpat = (char**)malloc(index*sizeof(char*));
    for (index = 0; groups[index]; index++) {
        savpat[index] = (char*)malloc(strlen(groups[index])+1);
        strcpy(savpat[index], groups[index]);
        pat[index] = 0;
        LIMnggetlist(&pat[index], savpat[index]);
    }
    pat[index] = 0;
    savpat[index] = 0;

    /* check for multiple group matches */
    for (index = 0; grplist[index] && !bad; index++) {
        ng[0] = grplist[index];
        newmatch = 0;
        for (index2 = 0; pat[index2] && !newmatch; index2++)
            if (LIMpermmatch(pat[index2], ng))
                newmatch = index2+1;
        if (nummatch == -1)
            nummatch = newmatch;
        else if (nummatch != newmatch)
            bad = 1;
    }

    /* free the patterns */
    for (index = 0; groups[index]; index++) {
        free((void*)savpat[index]);
        free((void*)pat[index]);
    }
    free((void*)savpat);
    free((void*)pat);

    free((void*)savto);
    free((void*)grplist);
    if (bad) {
        if (LIMgeterrmsg(filt, "G"))
            ERRORseterror(globfilt, LIMgeterrmsg(filt, "G"));
        else
            ERRORseterror(globfilt, "Crossposted between mutually exclusive hierarchies.");
        LIMblock(lim, "G");
    }
}
#endif

#if	defined(DO_FILT_EMILY)
int
LIMemily(filt, name, param)
    FILT *filt;
    char *name;
    LIMconfpart *param;
{
    int index = 1;
    char tok[80], *iter;
    int line = 0;

    if (!param || !param[0].lines || !param[0].lines[0] || param[1].lines)
        return(0);
    iter = 0;
    if (!(iter = LIMgetparttok(iter, tok, &line, param)))
        return(0);
    FILTaddstring(filt, sLIMemfile, tok);
    if (iter = LIMgetparttok(iter, tok, &line, param))
        return(0);
    return(1);
}

void
LIMpostnews(globfilt, filt, lim)
    FILT *globfilt, *filt;
    LIM *lim;
{
    char *stat;
    char *iter, *lparen;
    char *savng, **nglist, *curng[2];
    char *savpat, **pat;
    char *newng;
    char *emgroups, emfilename[255];
    FILE *logfile;
    int fd;
    char err[1024];

    if (!LIMparmloaded(filt, "E"))
        return;
    /* three possibilities:
     * non-existant, empty "", or deny "!" */
    stat = LIMstatdata(lim, "E");
    iter = stat;

    if (stat && !*stat)
        return;

    /* We've got to send out rules. */
    if (!stat)
        LIMaddsync(lim, "E", LIMsyncemily);
    if (LIMgeterrmsg(filt, "E"))
        ERRORseterror(globfilt, LIMgeterrmsg(filt, "E"));
    else
        ERRORseterror(globfilt, "Permission denied.");
    LIMblock(lim, "E");
}

void
LIMsyncemily(filt, locfilt, lim)
    FILT *filt, *locfilt;
    LIM *lim;
{
    int fd;
    FILE *logfile;
    char emfilename[256];

    if (FILTgetfile(filt, sMSGlog))
        return;
    fd = CUPfile(FILTgetuser(filt), 0, emfilename, sizeof(emfilename),
      _PATH_TMP);
    logfile = fdopen(fd, "w");
    FILTaddfile(filt, sMSGlog, emfilename, logfile);
    FILTaddstring(filt, sLIMrulefile, FILTgetstring(locfilt, sLIMrulefile));
    LIMaddfinish(lim, "E", LIMfinemily);
}

void
LIMfinemily(filt, locfilt, lim)
    FILT *filt, *locfilt;
    LIM *lim;
{
    char *filename;

    filename = FILTgetstring(locfilt, sLIMemfile);
    CUPsetfilename(locfilt, FILTgetfname(filt, sMSGlog));
    LIMsendmessage(locfilt, filename);
    unlink(FILTgetfname(filt, sMSGlog));
    FILTdelfile(filt, sMSGlog);
}
#endif

void
LIMsendmessage(filt, filename)
    FILT *filt;
    char *filename;
{
    FILE *emtext;
    char *text;
    char cmdline[255];
    long start, end;
    int index;

    if (!(emtext = fopen(filename, "r")))
        return;
    fgets(cmdline, sizeof(cmdline), emtext);
    /* strip '\n' */
    cmdline[strlen(cmdline)-1] = 0;
    start = ftell(emtext);
    fseek(emtext, 0, SEEK_END);
    end = ftell(emtext);
    text = (char*)malloc(end-start+1);
    fseek(emtext, start, SEEK_SET);
    fread(text, (end-start), 1, emtext);
    fclose(emtext);
    text[end-start] = 0;
    NOTIFYsendmail(filt, cmdline, text);
    free((void*)text);
}

typedef struct _LIMdbdata {
    char *name;
    LIMlimdecl limdecl;
} LIMdbdata;

int
LIMdb(filt, name, param)
    FILT *filt;
    char *name;
    LIMconfpart *param;
{
    LIMdbdata data;
    char *iter, tok[80];
    int line;

    data.limdecl.run = 0;
    data.limdecl.filt = 0;
    data.limdecl.children = 0;
    iter = 0;
    if (!(iter = LIMgetparttok(iter, tok, &line, &param[0])))
        return(0);
    data.name = (char*)malloc(strlen(tok)+1);
    strcpy(data.name, tok);
    if (iter = LIMgetparttok(iter, tok, &line, &param[0])) {
        free((void*)data.name);
        return(0);
    }
    LIMparselimitreal(&data.limdecl, param+1);
    line = 0;
    iter = 0;
    FILTadddata(filt, kFILTldecltype, sLIMdbname, (void*)&data,
        sizeof(data));
    return(1);
}

void
LIMrundb(globfilt, filt, lim)
    FILT *globfilt, *filt;
    LIM *lim;
{
    size_t s;
    LIMdbdata *data;
    LIM *nlim;

    if (!LIMparmloaded(filt, "DB"))
        return;
    data = FILTgetdata(filt, kFILTldecltype, sLIMdbname, &s);
    if (!data)
        return;
    nlim = LIMopendb(filt, data->name, lim);
    LIMbuildfilt(filt, nlim, &data->limdecl);
    LIMchecklim(globfilt, nlim);
}

int
LIMerrmsg(filt, name, param)
    FILT *filt;
    char *name;
    LIMconfpart *param;
{
    char *iter, tok[80], errname[80];
    char *lq, *rq;
    int i;
    int line = 0;

    iter = 0;
    if (!(iter = LIMgetparttok(iter, tok, &line, &param[0])))
        return(0);
    for (i = 0; LIMloadlist[i].name; i++)
        if (STRtokinlist(tok, LIMloadlist[i].name))
            break;
    if (!LIMloadlist[i].name)
        return(0);
    if (!(lq = strchr(param[1].lines[0], '"')))
        return(0);
    if (!(rq = strchr(lq+1, '"')))
        return(0);
    sprintf(errname, sLIMerrmsg, LIMloadlist[i].name[0]);
    *rq = 0;
    FILTaddstring(filt, errname, lq+1);
    return(1);
}

char *
LIMgeterrmsg(FILT *filt, char *name)
{
    int i;
    char errbuf[80];

    for (i = 0; LIMloadlist[i].name; i++)
        if (STRtokinlist(name, LIMloadlist[i].name))
            break;
    if (!LIMloadlist[i].name)
        return(0);
    sprintf(errbuf, sLIMerrmsg, LIMloadlist[i].name[0]);
    return(FILTgetstring(filt, errbuf));
}
