#ifndef __GUCS_INTERNAL_H
#define __GUCS_INTERNAL_H

#include "gucs_err.h"

typedef struct _GUCS_cf_buffer GUCS_cf_buffer;
typedef struct _GUCS_cf_position GUCS_cf_position;

typedef struct
{
	GUCS_cf_position *start;
	GUCS_cf_position *cur;
} GUCS_cf_expression;

#define GUCS_PARSE_BEGIN	-1
typedef int (*GUCS_parse_f)(GUCS_cf_expression *, int, void *, int, void **,
                            GUCS_error_type **);
typedef void (*GUCS_free_f)(int, void *);

typedef struct
{
	GUCS_parse_f parse_f;
	GUCS_free_f free_f;
	void *parse_data;
	int parse_data_len;
	void *ret;
	int ret_len;
} GUCS_parse_el;

/* file parsing routines */
extern GUCS_cf *GUCS_pos_file(GUCS_cf_position *pos);

/* diagnostic */
extern int GUCS_pos_lineno(GUCS_cf_position *pos);
extern char *GUCS_pos_filename(GUCS_cf_position *pos);

/* position in the file */
extern GUCS_cf_position *GUCS_file_start(GUCS_cf *cf);
extern GUCS_cf_position *GUCS_pos_copy(GUCS_cf_position *pos);
extern void GUCS_pos_free(GUCS_cf_position *pos);
extern int GUCS_pos_eof(GUCS_cf_position *pos);

/* part of the line */
extern int GUCS_pos_peek(GUCS_cf_position *pos, void *buf, size_t size);
extern void GUCS_pos_advance(GUCS_cf_position *pos, int length);

/* parse routines */
extern GUCS_cf_expression *GUCS_exp_alloc(GUCS_cf_position *start);
extern void GUCS_exp_free(GUCS_cf_expression *exp);
extern GUCS_cf_position *GUCS_exp_start(GUCS_cf_expression *exp);
extern GUCS_cf_position *GUCS_exp_cur(GUCS_cf_expression *exp);

extern void GUCS_skip(GUCS_cf_position *pos, GUCS_parse_f parse);

extern GUCS_error_type *GUCS_attempt_parse(GUCS_cf_position *, GUCS_parse_el *);
extern GUCS_error_type *GUCS_parse(GUCS_cf_position *, GUCS_parse_el **, int *);
extern void GUCS_reset_parse(GUCS_parse_el *els);
extern int GUCS_seek_eoln(GUCS_cf_position *pos);

extern int GUCS_parse_comment(GUCS_cf_expression *, int, void *, int, void **,
                              GUCS_error_type **);
extern int GUCS_parse_space(GUCS_cf_expression *, int, void *, int, void **,
                            GUCS_error_type **);
extern int GUCS_parse_to_eoln(GUCS_cf_expression *, int, void *, int, void **,
                              GUCS_error_type **);
extern int GUCS_parse_string(GUCS_cf_expression *, int, void *, int, void **,
                             GUCS_error_type **);
extern int GUCS_parse_number(GUCS_cf_expression *, int, void *, int, void **,
                             GUCS_error_type **);
extern int GUCS_parse_token(GUCS_cf_expression *, int, void *, int, void **,
                            GUCS_error_type **);

/* specifically for GUCS-generated code. */
typedef GUCS_error_type *(*GUCS_realm_parser)(GUCS_cf_position *, void *,
                                              int, void*);

extern GUCS_error_type *GUCS_include(GUCS_cf *cf, GUCS_realm_parser parser,
                                     void *data, void *state);
extern GUCS_error_type *GUCS_parse_err(GUCS_cf_position *, GUCS_cf_position *,
                                       int);

extern GUCS_parse_el GUCS_lbrace[];
extern GUCS_parse_el GUCS_rbrace[];
extern GUCS_parse_el GUCS_blank[];

#endif
