#include <sys/stat.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>

#include "filtpaths.h"
#include "filter.h"
#include "proto.h"
#include "catchup.h"

int
CUPfile(user, delay, fname, len, confval)
    char *user;
    int delay;
    char *fname;
    size_t len;
    char *confval;
{
    int ret;

    /* open is atomic, but there's a bit of a race between getting a
     * unique filename and opening the file.  Therefore, keep trying
     * to open the unique filename until we've finally got a valid fd */
    while (1) {
        CUPfilename(user, delay, fname, len, confval);
        ret = open(fname, O_RDWR | O_CREAT | O_EXCL, 0600);
        if ((ret < 0) && (errno != EEXIST))
            return(ret);
        else if (ret >= 0)
            return(ret);
    }
    return(-1);
}

void
CUPfilename(user, delay, fname, len, dir)
    char *user;
    int delay;
    char *fname;
    size_t len;
    char *dir;
{
    int index, ret;
    char buf[255];
    struct stat st;

    strncpy(fname, dir, len);
    fname[len-1] = 0;
    while(fname[strlen(fname)-1] == '/')
        fname[strlen(fname)-1] = 0;
    sprintf(buf, "%s/%s.%01d", fname, user, delay);

    /* get a unique filename in this directory */
    index = 0;
    while (1) {
        sprintf(fname, "%s.%d", buf, index);
        ret = stat(fname, &st);
        if ((ret < 0) && (errno != EEXIST))
            break;
        index++;
    }
}

int
CUPparsefname(fname, user, len, delay)
    char *fname;
    char *user;
    size_t len;
    int *delay;
{
    char *basename, *dot;
    char buf[255];
    int index;

    basename = strrchr(fname, '/');
    if (basename)
        basename++;
    else
        basename = fname;
    strcpy(buf, basename);
    basename = buf;

    /* Do this backwards since username could be anything */
    if (!(dot = strrchr(basename, '.')))
        return (1);
    /* Make sure .<unique> is all number */
    *dot++ = 0;
    if (!*dot)
        return(1);
    for (index = 0; dot[index]; index++)
        if ((dot[index]<'0') || (dot[index]>'9'))
            return(1);

    /* Make sure .<delay> is all number */
    dot = strrchr(basename, '.');
    *dot++ = 0;
    if (!*dot)
        return(1);
    for (index = 0; dot[index]; index++)
        if ((dot[index]<'0') || (dot[index]>'9'))
            return(1);
    *delay = atoi(dot);

    /* User gets the rest */
    if (!*basename)
        return(1);
    strncpy(user, basename, len);
    return(0);
}

char *
CUPgetfilename(filt)
    FILT *filt;
{
    return(FILTgetstring(filt, sCUPfname));
}

void
CUPsetfilename(filt, name)
    FILT *filt;
    char *name;
{
    FILTaddstring(filt, sCUPfname, name);
}
