#include "dep.h"
#include <string.h>

deplist dep_glob = 0;

static depstruc *find_dep_real(deplist, char*);

/* add a depstruc to a list (duh) */
static int dep_add_list(deplist *list, depstruc *struc)
{
    int i;
    deplist new;

    if (!*list) {
        /* initialize a new list */
        *list = (deplist)malloc(2*sizeof(depstruc*));
        if (!*list)
            return(-1);
        (*list)[0] = struc;
        (*list)[1] = 0;
        struc->deplist = *list;
        return(0);
    }
    /* create a new list with what's in the old */
    for (i = 0; (*list)[i]; i++)
        ;
    new = (deplist)realloc(*list, (2+i)*sizeof(depstruc*));
    if (!new)
        return(-1);
    *list = new;
    new[i] = struc;
    new[i+1] = 0;
    for (i = 0; new[i]; i++)
        new[i]->deplist = new;
    return(0);
}

/* create a dependancy that hasn't been created before */
depstruc *new_dep(deplist *list, char *name)
{
    depstruc *new;

    if (name && (new = find_dep(*list, name)))
        return(new);
    
    new = (depstruc*)malloc(sizeof(depstruc));
    if (!new)
        return(0);

    if (name) {
        new->name = (char*)malloc(strlen(name)+1);
        if (!new->name) {
            free((void*)new);
            return(0);
        }
        strcpy(new->name, name);
    } else
        new->name = 0;
    new->data = 0;
    new->children = 0;
    new->action = 0;
    /* add this guy to the list */
    if (dep_add_list(list, new)) {
        if (new->name)
            free((void*)new->name);
        free((void*)new);
        return(0);
    }
    return(new);
}

/* go through a list of children touching only those that haven't been touched
 * before */
static void dep_touch_real(depstruc *struc, deplist *touched, void *data)
{
    int i;

    if (struc->action)
        struc->action(struc, data);
    if (struc->children)
        for (i = 0; struc->children[i]; i++)
            if (!find_dep_real(*touched, struc->children[i]->data)) {
                dep_add_list(touched, struc->children[i]);
                dep_touch_real(struc->children[i], touched, data);
            }
}

/* mark a dep for update, as well as everything that depends on it */
void dep_touch(depstruc *struc, void *data)
{
    deplist touched = 0;

    dep_touch_real(struc, &touched, data);
    if (touched)
        free((void*)touched);
}

/* find a depstruc associated with a piece of data */
static depstruc *find_dep_real(deplist list, char *name)
{
    int i;

    if (!list)
        return(0);
    for (i = 0; list[i]; i++)
        if (list[i]->name && !strcmp(list[i]->name, name))
            return(list[i]);
    return(0);
}

/* exported function */
depstruc *find_dep(deplist list, char *name)
{
    return(find_dep_real(list, name));
}

/* add a dependancy to a parent */
int add_dep(depstruc *parent, depstruc *child)
{
    return(dep_add_list(&parent->children, child));
}
