#ifndef __LIMITFILE_H
#define __LIMITFILE_H

#include <sys/types.h>
#include <stdio.h>
#include "filter.h"

#define kLIMnotblocked	0
#define kLIMblockthis	1
#define kLIMblockelse	2

#define kLIMopand 0
#define kLIMopor 1
#define kLIMopnot 2

typedef struct {} LIM;
typedef struct {
    char **lines;
} LIMconfpart;

extern char *LIMgetparttok(char*, char*, int*, LIMconfpart*);

typedef int (*LIMparsefunc)(char*, LIMconfpart*);

typedef int (*LIMloadfunc)(FILT*, char*, LIMconfpart*);
typedef void (*LIMdepfunc)(FILT*, LIMconfpart*, int*);
typedef void (*LIMlimfunc)(FILT*, FILT*, LIM*);
typedef void (*LIMsyncfunc)(FILT*, FILT*, LIM*);
typedef void (*LIMfinishfunc)(FILT*, FILT*, LIM*);

#define functype(ftype, tname) \
    typedef struct { \
        char **name; \
        ftype func; \
    } tname

functype(LIMparsefunc, LIMparse);
functype(LIMdepfunc, LIMdep);
functype(LIMloadfunc, LIMload);
#undef functype

typedef struct {
    char **name;
    int type;
} LIMop;

extern LIMparse LIMparselist[];
extern LIMdep LIMdeplist[];
extern LIMop LIMoplist[];
extern LIMload LIMloadlist[];
extern LIMlimfunc LIMlimlist[];

extern int LIMloadconfig(char*);
extern void LIMtrackpost(FILT*);
extern void LIMfinish(FILT*);
extern int LIMshouldsend(FILT*);

/* support for nested limit declarations */

typedef struct _LIMtestlist {
    int pass;
    int op;
    int not;
    struct _LIMtestlist *next;
} LIMtestlist;

typedef struct _LIMlimdecl {
    LIMtestlist *run;
    FILT *filt;
    /* allow for nesting limits */
    struct _LIMlimdecl **children;
} LIMlimdecl;

extern int LIMparselimitreal(LIMlimdecl*, LIMconfpart*);
extern void LIMchecklim(FILT*, LIM*);
extern int LIMshouldrun(LIMtestlist*);

/* data accessor functions.  Determine what data goes into the persistant
 * database */

extern int LIMparmloaded(FILT*, char*);
extern LIM *LIMopendb(FILT*, char*, LIM*);
extern FILT *LIMbuildfilt(FILT*, LIM*, LIMlimdecl*);

/* dynamic data */
extern char **LIMdata(LIM*, char*); /* state, name */
extern char *LIMnewdata(LIM*, char*); /* state, name */
extern void LIMsetdata(LIM*, char*, char*); /* state, name, new */
extern void LIMcleardata(LIM*, char*); /* state, name */
extern void LIMresetdata(LIM*, char*); /* state, name */

/* static data */
extern char *LIMstatdata(LIM*, char*); /* state, name */
extern char *LIMnewstatdata(LIM*, char*); /* state, name */
extern void LIMsetstatdata(LIM*, char*, char*); /* state, name, newstr */
extern void LIMclearstatdata(LIM*, char*); /* state, name */
extern void LIMresetstatdata(LIM*, char*); /* state, name */

/* post behavior */
extern void LIMblock(LIM*, char*); /* state, name */
extern void LIMclearblock(LIM*, char*); /* state, name */
extern int LIMisblock(LIM*, char*); /* state, name */

/* Register postprocessors.  These take over after the post has been logged
 * or whatnot, and finish sending out notification or what have you. */
extern void LIMaddfinish(LIM*, char*, LIMfinishfunc);
extern void LIMresetfinish(LIM*, char*, LIMfinishfunc);
extern void LIMaddsync(LIM*, char*, LIMsyncfunc);
extern void LIMresetsync(LIM*, char*);

#endif
