#include <stdio.h>
#include <errno.h>
#include <string.h>

typedef struct _host_s {
    char *hostname;
    int articles;
    struct _host_s **feeders;
} host_t;

#define INHOST "ZCZC begin inhosts 2.5 "
#define INPATH "ZCZC begin inpaths 2.5 "
#define ENDPATH "ZCZC end inpaths "

host_t **hosts;

host_t *findhost(host_t**, char*);
host_t **addhost(host_t**, host_t*);

int main(int argc, char *argv[]) {
    int i, j;
    char buf[4096]; /* Really Big Buffer for the longest possible lines in a
		     * file. */
    FILE *f;
    char *iter, *number, *bufiter;
    host_t *new, *feeder;
    int tottraffic;
    char *hostname;
    int columns;

    hosts = 0;
    buf[sizeof(buf) - 1] = '\0';
    tottraffic = 0;
    for (i = 1; i < argc; i++) {
	if (!(f = fopen(argv[i], "r"))) {
	    fprintf(stderr, "Could not open file %s: %s\n", argv[i],
	      strerror(errno));
	    continue;
	}
	if (!fgets(buf, sizeof(buf)-1, f)) {
	    fprintf(stderr, "%s: Empty file.\n", argv[i]);
	    fclose(f);
	    continue;
	}
	if (strncmp(buf, INHOST, sizeof(INHOST)-1) != 0) {
	    /* reading total host volume */
	    fprintf(stderr, "%s: Not an inpaths 2.5 file.\n", argv[i]);
	    fclose(f);
	    continue;
	}
	buf[strlen(buf)-1] = '\0';
	bufiter = buf+sizeof(INHOST)-1;
	if (!hostname)
	    hostname = strdup(strsep(&bufiter, " \t\n"));
	else
	    strsep(&bufiter, " \t\n");
	strsep(&bufiter, " \t\n");
	tottraffic += atoi(strsep(&bufiter, " \t\n"));
	while (iter = fgets(buf, sizeof(buf)-1, f)) {
	    if (strncmp(buf, INPATH, sizeof(INPATH)-1) == 0)
		/* reading real path lines */
		break;
	    buf[strlen(buf)-1] = '\0';
	    bufiter = buf;
	    number = strsep(&bufiter, " \t");
	    iter = strsep(&bufiter, " \t");
	    new = findhost(hosts, iter);
	    if (!new) {
		new = (host_t*) malloc(sizeof(host_t));
		hosts = addhost(hosts, new);
		new->articles = 0;
		new->hostname = strdup(iter);
		new->feeders = 0;
	    }
	    new->articles += atoi(number);
	}
	if (!iter) {
	    fprintf(stderr, "%s: Unexpected EOF, output may be messed up.\n", argv[i]);
	    fclose(f);
	    continue;
	}
	while (iter = fgets(buf, sizeof(buf)-1, f)) {
	    if (strncmp(buf, ENDPATH, sizeof(ENDPATH)-1) == 0)
		/* reading real path lines */
		break;
	    buf[strlen(buf)-1] = '\0';
	    bufiter = buf;
	    iter = strsep(&bufiter, " \t");
	    new = findhost(hosts, iter);
	    if (!new) {
		new = (host_t*) malloc(sizeof(host_t));
		hosts = addhost(hosts, new);
		new->articles = 0;
		new->hostname = strdup(iter);
		new->feeders = 0;
	    }
	    iter = strsep(&bufiter, " \t");
	    while (bufiter && *bufiter) {
		number = strsep(&bufiter, " \t");
		strsep(&bufiter, " \t");
		iter = strsep(&bufiter, " \t");
		strsep(&bufiter, " \t");
		feeder = findhost(new->feeders, iter);
		if (!feeder) {
		    feeder = (host_t*) malloc(sizeof(host_t));
		    new->feeders = addhost(new->feeders, feeder);
		    feeder->hostname = strdup(iter);
		    feeder->feeders = 0;
		    feeder->articles = 0;
		}
		feeder->articles += atoi(number);
	    }
	}
	if (!iter) {
	    fprintf(stderr, "%s: Unexpected EOF, output may be messed up.\n", argv[i]);
	    fclose(f);
	    continue;
	}
	if (fgets(buf, sizeof(buf)-1, f)) {
	    fprintf(stderr, "%s: Unexpected data after end of inpaths.  Are you sure this is a good file?\n", argv[i]);
	    continue;
	}
    }
    printf("%s%s 2 %d 0.0\n", INHOST, hostname, tottraffic);
    for (i = 0; hosts[i]; i++)
	printf("%d\t%s\n", hosts[i]->articles, hosts[i]->hostname);
    printf("%s%s 2 %d 0.0\n", INPATH, hostname, tottraffic);
    for (i = 0; hosts[i]; i++) {
	if (!hosts[i]->feeders)
	    continue;
	columns = strlen(hosts[i]->hostname)+3;
	printf("%s H", hosts[i]->hostname);
	for (j = 0; hosts[i]->feeders[j]; j++) {
	    columns += strlen(hosts[i]->feeders[j]->hostname)+4;
	    if (columns > 79) {
		printf("\n%s H", hosts[i]->hostname);
		columns = strlen(hosts[i]->hostname)+3;
	    }
	    printf(" %d Z %s U", hosts[i]->feeders[j]->articles,
	      hosts[i]->feeders[j]->hostname);
	}
	printf("\n");
    }
    printf("%s%s\n", ENDPATH, hostname);
}

host_t *findhost(host_t **list, char *name)
{
    int i;

    if (!list)
	return(0);
    for (i = 0; list[i]; i++)
	if (!strcmp(list[i]->hostname, name))
	    return(list[i]);
    return(0);
}

host_t **addhost(host_t **list, host_t *host)
{
    host_t **ret;
    int i;

    if (!list) {
	ret = (host_t**) malloc(sizeof(host_t*)*2);
	ret[0] = host;
	ret[1] = 0;
	return(ret);
    }
    for (i = 0; list[i]; i++)
	continue;
    ret = (host_t**) realloc((void*)list, (i+2)*sizeof(host_t*));
    ret[i] = host;
    ret[i+1] = 0;
    return(ret);
}
