/* $Id: corp-ftp.c,v 1.1.1.1.2.2 1997/09/11 19:36:52 aidan Exp $ */
/* $Log: corp-ftp.c,v $
/* Revision 1.1.1.1.2.2  1997/09/11 19:36:52  aidan
/* Now supports logging into the default host if the IP isn't in IPADDRS
/* */

#include "corp-ftp.h"
#include "strings.h"
#include "pathnames.h"

#include <netdb.h>
#include <string.h>
#include <syslog.h>
#include <varargs.h>
#include <pwd.h>
#include <stdio.h>

userlist_t uids;

int corp_get_info(const char *ip_num, char *dir)
{
   FILE *ip_addrf;
   char Line[MAX_LINE_LEN], *line=Line, Word[MAX_LINE_LEN], *word=Word, *comma, *hyphen;
   int flag = 0, addr_len, index;

   /* make sure uids are 0 on error */
   uids.data = 0;
   uids.numdata = 0;
   if ((ip_addrf = fopen(_PATH_IP_DIR, "r")) == NULL) {
      syslog(LOG_WARNING, "couldn't open IP map file `%s' for reading!", _PATH_IP_DIR);
      return -1;
   }

   while (get_line(ip_addrf, line) != -1) {
      line += get_word(line, word);
      if (!strcmp(word, ip_num)) {
	 flag = 1;
	 break;
      }
      line=Line;
   }
   fclose(ip_addrf);

   if (!flag) {
      syslog(LOG_NOTICE, "%s not found in IP->directory mapping", ip_num);
      return -1;
   }

   if (get_word(line, dir) == -1) {
      syslog(LOG_ALERT, "no directory entry for IP Address %s", ip_num);
      return -1;
   }

   /* now read in the alloted uids */
   if (get_word(line+strlen(dir)+1, word) != -1) {
      /* the lines have the form '1000,1200-1500,31000'.  The commas are
       * seperators for the items, so we allocate storage for numcommas+1 */
      fflush(stdout);
      comma = word;
      uids.numdata = 1;
      while(comma = strchr(comma, ',')) {
         comma++;
         uids.numdata++;
      }
      uids.data = (user_t*)malloc(sizeof(user_t)*uids.numdata);
      if (!uids.data)
         return(-1);

      /* now fill in the data, barfing and cleaning up if it's not a number */
      /* TODO: barf and clean up if a uid is not a number */
      comma = word;
      for (index = 0; index<uids.numdata; index++) {
         if (comma) {
            comma = strchr(word, ',');
            if (comma)
               *comma++ = 0;
         }
         if (hyphen = strchr(word, '-')) {
            /* we've got an entire range of uids */
            *hyphen = 0;
            uids.data[index].low = atoi(word);
            word = hyphen+1;
            uids.data[index].high = atoi(word);
         } else {
            /* only one uid.  Low and high are the same */
            uids.data[index].low = uids.data[index].high = atoi(word);
         }
         fflush(stdout);
         word = comma;
      }
   }

   return 0;
}

int
corp_check_uid(uid_t id)
{
    int index;

    if (id == 0)
        return(-1);
    for (index = 0; index < uids.numdata; index++)
        if ((id>=uids.data[index].low)&&(id<=uids.data[index].high))
            return(0);
    return(-1);
}

struct passwd *
corp_getpwnam(name)
    const char *name;
{
   static struct passwd save;
   register struct passwd *p;
   char *sgetsave(char *s);

   /* getpwnam on BSD searches in a spwd.db file.  We don't use that. */
   FILE *pwfile;
   char buf[255];
   char *colon, *iter;
   int found;

   pwfile = fopen("/etc/passwd", "r");
   if (!pwfile)
      return(NULL);
   found = 0;
   while ((!found) && fgets(buf, sizeof(buf), pwfile)) {
      buf[strlen(buf)-1] = 0; /* clean off the \n */
      if (!(colon = strchr(buf, ':'))) {
         fclose(pwfile);
         return(NULL);
      }
      *colon = 0;
      if (!strcmp(buf, name))
         found = 1;
   }
   fflush(stdout);
   if (!found)
      return(NULL);
   p = (struct passwd*)malloc(sizeof(struct passwd));
   p->pw_name = buf;
   iter = colon+1;
   if (!(colon = strchr(iter, ':'))) {
      free((void*)p);
      return(NULL);
   }
   *colon = 0;
   p->pw_passwd = iter;
   iter = colon+1;
   if (!(colon = strchr(iter, ':'))) {
      free((void*)p);
      return(NULL);
   }
   *colon = 0;
   save.pw_uid = atoi(iter);
   iter = colon+1;
   if (!(colon = strchr(iter, ':'))) {
      free((void*)p);
      return(NULL);
   }
   *colon = 0;
   p->pw_gecos = iter;
   iter = colon+1;
   if (!(colon = strchr(iter, ':'))) {
      free((void*)p);
      return(NULL);
   }
   *colon = 0;
   p->pw_dir = iter;
   iter = colon+1;
   p->pw_shell = iter;
   save.pw_name = sgetsave(p->pw_name);
   save.pw_passwd = sgetsave(p->pw_passwd);
   save.pw_dir = sgetsave(p->pw_dir);
   save.pw_shell = sgetsave(p->pw_shell);
   save.pw_gecos = sgetsave(p->pw_gecos);
   return(&save);
}
