# include files for generated .c file.
%include {
#include "config.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <netdb.h>

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "paths.h"
}

# stuff appended to generated .h file.
%export {

typedef struct _catchup_dir_cfg_s
{
	char *dname;
	int check;
} catchup_dir_cfg_s;

typedef struct _catchup_config_s
{
	char *pidfile;
	char *posterror;
	catchup_dir_cfg_s hold, del;
	int detach;
	struct sockaddr_in server;
} catchup_config_s;

extern void catchup_config_set_host(catchup_config_s *, char *);
extern void catchup_config_set_port_string(catchup_config_s *, char *);
extern void catchup_config_set_port_number(catchup_config_s *, short);
extern void catchup_config_defaults(catchup_config_s *);
extern void catchup_config_dispose(catchup_config_s *);
extern void catchup_config_merge(catchup_config_s *, catchup_config_s *,
                                 int, int);

}

# local type/function declarations.
%declarations {

void catchup_config_init(catchup_config_s *);

}

%abstract dir (catchup_dir_cfg_s *dir_cfg, char *descr) {
	state {
		catchup_dir_cfg_s *${dir_cfg};
		char *${descr};
	}

	init_code {
		${dir_cfg} = dir_cfg;
		${descr} = descr;
	}

	attr dir <string:dir> {
		if (${dir_cfg}->dname != (char *) 0)
		{
			fprintf(stderr, "already specified %s dir.\n",
			        ${descr});
			exit(1);
		}
		${dir_cfg}->dname = strdup(@{dir});
	}
}

%unit catchup {
	state {
		catchup_config_s ${local_config};
		catchup_config_s *${config};
		int ${got_addr};
		int ${got_detach};
	}

	init_code {
		${config} = (catchup_config_s *) GUCS_data;
		catchup_config_init(&${local_config});
		${got_addr} = 0;
		${got_detach} = 0;
	}

	finish_code {
		/* override default config */
		catchup_config_merge(${config}, &${local_config},
		                     ${got_addr}, ${got_detach});
	}

	attr pid-file: <string:file> {
		if (${local_config}.pidfile != (char *) 0)
		{
			fprintf(stderr, "already specified pid file.\n");
			exit(1);
		}
		${local_config}.pidfile = strdup(@{file});
	}

	attr post-error: <string:file> {
		if (${local_config}.posterror != (char *) 0)
		{
			fprintf(stderr, "already specified post-error location.\n");
			exit(1);
		}
		${local_config}.posterror = strdup(@{file});
	}

	attr detach {
		${local_config}.detach = 1;
		${got_detach} = 1;
	}

	attr no-detach {
		${local_config}.detach = 0;
		${got_detach} = 1;
	}

	parse spool {
		&{Parse:dir(&${local_config}.hold, "spool")};
	}

	parse del {
		&{Parse:dir(&${local_config}.del, "del")};
	}

	realm connect {
		init_code {
			${got_addr} = 1;
		}

		attr host: <string:ip> {
			catchup_config_set_host(&${local_config}, @{ip});
		}

		attr port: <number:port> {
			catchup_config_set_port_number(&${local_config},
			                               @{port});
		}

		attr port: <string:port> {
			catchup_config_set_port_string(&${local_config},
			                               @{port});
		}
	}
}

%code {

void
catchup_config_set_host(catchup_config_s *config, char *host)
{
	struct hostent *hent;

	config->server.sin_addr.s_addr = inet_addr(host);
	if (config->server.sin_addr.s_addr == (u_int32_t) -1)
	{
		hent = gethostbyname(host);
		if (hent == (struct hostent *) 0)
		{
			fprintf(stderr, "gethostbyname(%s) failed.\n", host);
			exit(1);
		}
		memcpy(&config->server.sin_addr.s_addr, hent->h_addr_list[0],
		       sizeof(config->server.sin_addr.s_addr));
	}
}

void
catchup_config_set_port_string(catchup_config_s *config, char *port)
{
	struct servent *sent;

	sent = getservbyname(port, "tcp");
	if (sent == (struct servent *) 0)
	{
		fprintf(stderr, "getservbyname(%s/tcp) failed.\n", port);
		exit(1);
	}
	config->server.sin_port = sent->s_port;
}

void
catchup_config_set_port_number(catchup_config_s *config, short port)
{
	config->server.sin_port = htons(port);
}

void
catchup_config_init(catchup_config_s *config)
{
	config->pidfile = (char *) 0;
	config->posterror = (char *) 0;
	config->hold.dname = (char *) 0;
	config->del.dname = (char *) 0;
	bzero(&config->server, sizeof(config->server));
	config->server.sin_family = AF_INET;
	config->detach = 0;
}

void
catchup_config_defaults(catchup_config_s *config)
{
	config->pidfile = strdup(CATCHUP_PID_FILE);
	config->posterror = strdup(SHOGUN_POSTERROR);
	config->hold.dname = strdup(SHOGUN_HOLD_DIR);
	config->del.dname = strdup(SHOGUN_DEL_DIR);
	catchup_config_set_host(config, "localhost");
	catchup_config_set_port_string(config, "nntp");
	config->detach = 1;
}

void
catchup_config_dispose(catchup_config_s *config)
{
	free((void *) config->pidfile);
	free((void *) config->posterror);
	free((void *) config->hold.dname);
	free((void *) config->del.dname);
}

void
catchup_config_merge(catchup_config_s *dest, catchup_config_s *src,
                      int got_addr, int got_detach)
{
#define CATCHUP_OVERRIDE_STRING(param) \
	if (src->param != (char *) 0) \
	{ \
		free(dest->param); \
		dest->param = src->param; \
	}

	CATCHUP_OVERRIDE_STRING(pidfile);
	CATCHUP_OVERRIDE_STRING(posterror);
	CATCHUP_OVERRIDE_STRING(hold.dname);
	CATCHUP_OVERRIDE_STRING(del.dname);
#undef CATCHUP_OVERRIDE_STRING

	if (got_addr)
		memcpy(&dest->server, &src->server, sizeof(dest->server));
	if (got_detach)
		dest->detach = src->detach;
}

}
